# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Test functions for utils.matrices """
import numpy as np

from ..matrices import (matrix_rank, full_rank, pos_recipr, recipr0)

from nose.tools import (assert_true, assert_equal, assert_false,
                        assert_raises)

from numpy.testing import (assert_almost_equal, assert_array_almost_equal)


def test_matrix_rank():
    # Full rank matrix
    assert_equal(4, matrix_rank(np.eye(4)))
    I=np.eye(4); I[-1,-1] = 0. # rank deficient matrix
    assert_equal(matrix_rank(I), 3)
    # All zeros - zero rank
    assert_equal(matrix_rank(np.zeros((4,4))), 0)
    # 1 dimension - rank 1 unless all 0
    assert_equal(matrix_rank(np.ones((4,))), 1)
    assert_equal(matrix_rank(np.zeros((4,))), 0)
    # accepts array-like
    assert_equal(matrix_rank([1]), 1)
    # Make rank deficient matrix
    rng = np.random.RandomState(20120613)
    X = rng.normal(size=(40, 10))
    X[:, 0] = X[:, 1] + X[:, 2]
    S = np.linalg.svd(X, compute_uv=False)
    eps = np.finfo(X.dtype).eps
    assert_equal(matrix_rank(X, tol=0), 10)
    assert_equal(matrix_rank(X, tol=S.min() - eps), 10)
    assert_equal(matrix_rank(X, tol=S.min() + eps), 9)


def test_full_rank():
    rng = np.random.RandomState(20110831)
    X = rng.standard_normal((40,5))
    # A quick rank check
    assert_equal(matrix_rank(X), 5)
    X[:,0] = X[:,1] + X[:,2]
    assert_equal(matrix_rank(X), 4)
    Y1 = full_rank(X)
    assert_equal(Y1.shape, (40,4))
    Y2 = full_rank(X, r=3)
    assert_equal(Y2.shape, (40,3))
    Y3 = full_rank(X, r=4)
    assert_equal(Y3.shape, (40,4))
    # Windows - there seems to be some randomness in the SVD result; standardize
    # column signs before comparison
    flipper = np.sign(Y1[0]) * np.sign(Y3[0])
    assert_almost_equal(Y1, Y3 * flipper)


def test_pos_recipr():
    X = np.array([2,1,-1,0], dtype=np.int8)
    eX = np.array([0.5,1,0,0])
    Y = pos_recipr(X)
    yield assert_array_almost_equal, Y, eX
    yield assert_equal, Y.dtype.type, np.float64
    X2 = X.reshape((2,2))
    Y2 = pos_recipr(X2)
    yield assert_array_almost_equal, Y2, eX.reshape((2,2))
    # check that lists have arrived
    XL = [0, 1, -1]
    yield assert_array_almost_equal, pos_recipr(XL), [0, 1, 0]
    # scalars
    yield assert_equal, pos_recipr(-1), 0
    yield assert_equal, pos_recipr(0), 0
    yield assert_equal, pos_recipr(2), 0.5


def test_recipr0():
    X = np.array([[2,1],[-4,0]])
    Y = recipr0(X)
    yield assert_array_almost_equal, Y, np.array([[0.5,1],[-0.25,0]])
    # check that lists have arrived
    XL = [0, 1, -1]
    yield assert_array_almost_equal, recipr0(XL), [0, 1, -1]
    # scalars
    yield assert_equal, recipr0(-1), -1
    yield assert_equal, recipr0(0), 0
    yield assert_equal, recipr0(2), 0.5



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Convenience functions and classes for statistics on images.

These functions and classes support the return of statistical test results from
iterations through data.

The basic container here is the RegressionOutput.  This does two basic things:

* via __call__, processes a result object from a regression to produce
  something, usually an array
* via slicing (__setitem__), it can store stuff, usually arrays.

We use these by other objects (see algorithms.statistics.fmri.fmristat) slicing
data out of images, fitting models to the data to create results objects, and
then passing them to these here ``RegressionOutput`` containers via call, to get
useful arrays, and then putting the results back into the ``RegressionOutput``
containers via slicing (__setitem__).
"""

__docformat__ = 'restructuredtext'

import numpy as np


def output_T(results, contrast, retvals=('effect', 'sd', 't')):
    """ Convenience function to collect t contrast results

    Parameters
    ----------
    results : object
        implementing Tcontrast method
    contrast : array
        contrast matrix
    retvals : sequence, optional
        None or more of strings 'effect', 'sd', 't', where the presence of the
        string means that that output will be returned.

    Returns
    -------
    res_list : list
        List of results.  It will have the same length as `retvals` and the
        elements will be in the same order as retvals
    """
    r = results.Tcontrast(contrast, store=retvals)
    returns = []
    for valname in retvals:
        if valname == 'effect':
            returns.append(r.effect)
        if valname == 'sd':
            returns.append(r.sd)
        if valname == 't':
            returns.append(r.t)
    return returns


def output_F(results, contrast):
    """
    This convenience function outputs the results of an Fcontrast
    from a regression

    Parameters
    ----------
    results : object
        implementing Tcontrast method
    contrast : array
        contrast matrix

    Returns
    -------
    F : array
        array of F values
    """
    return results.Fcontrast(contrast).F


def output_resid(results):
    """
    This convenience function outputs the residuals
    from a regression
    """
    return results.resid


class RegressionOutput(object):
    """
    A class to output things in GLM passes through arrays of data.
    """

    def __init__(self, img, fn, output_shape=None):
        """
        Parameters
        ----------
        img : ``Image`` instance
            The output Image
        fn : callable
            A function that is applied to a
            models.model.LikelihoodModelResults instance
        """
        self.img = img
        self.fn = fn
        self.output_shape = output_shape

    def __call__(self, x):
        return self.fn(x)

    def __setitem__(self, index, value):
        self.img[index] = value


class RegressionOutputList(object):
    """
    A class to output more than one thing
    from a GLM pass through arrays of data.
    """

    def __call__(self, x):
        return self.fn(x)

    def __init__(self, imgs, fn):
        """ Initialize regression output list

        Parameters
        ----------
        imgs : list
            The list of output images
        fn : callable
            A function that is applied to a
            models.model.LikelihoodModelResults instance
        """
        self.list = imgs
        self.fn = fn

    def __setitem__(self, index, value):
        self.list[index[0]][index[1:]] = value


class TOutput(RegressionOutputList):
    """
    Output contrast related to a T contrast from a GLM pass through data.
    """
    def __init__(self, contrast, effect=None, sd=None, t=None):
        # Returns a list of arrays, being [effect, sd, t] when all these are not
        # None
        # Compile list of desired return values
        retvals = []
        # Set self.list to contain selected input catching objects
        self.list = []
        if not effect is None:
            retvals.append('effect')
            self.list.append(effect)
        if not sd is None:
            retvals.append('sd')
            self.list.append(sd)
        if not t is None:
            retvals.append('t')
            self.list.append(t)
        # Set return function to return selected inputs
        self.fn = lambda x: output_T(x, contrast, retvals)


def output_AR1(results):
    """
    Compute the usual AR(1) parameter on
    the residuals from a regression.
    """
    resid = results.resid
    rho = np.add.reduce(resid[0:-1]*resid[1:] / np.add.reduce(resid[1:-1]**2))
    return rho





#!/usr/bin/env python
""" Fix sphinx latex output for longtable
"""
import re
import sys

lt_LL = re.compile(
    r"longtable}{(L+)}")

def replacer(match):
    args =  '|' + 'l|' * len(match.groups()[0])
    return "longtable}{%s}" % args


if len(sys.argv) != 2:
    raise RuntimeError("Enter path to tex file only")
file_path = sys.argv[1]

unfixed_tex = open(file_path,'rt').readlines()
write_file = open(file_path, 'wt')
for line in unfixed_tex:
    line = lt_LL.sub(replacer, line, 1)
    write_file.write(line)



import numpy as np

from ..polyaffine import PolyAffine
from ..affine import Affine


def random_affine():
    T = np.eye(4)
    T[0:3, 0:4] = np.random.rand(3, 4)
    return T


def id_affine():
    return np.eye(4)


NCENTERS = 5
NPTS = 100

centers = [np.random.rand(3) for i in range(NCENTERS)]
raf = random_affine()
affines = [raf for i in range(NCENTERS)]
#affines = [id_affine() for i in range(NCENTERS)]
sigma = 1.0
xyz = np.random.rand(NPTS, 3)

# test 1: crach test create polyaffine transform
T = PolyAffine(centers, affines, sigma)

# test 2: crash test apply method
t = T.apply(xyz)

# test 3: check apply does nice job
c = np.array(centers)
tc = T.apply(c)
qc = np.array([np.dot(a[0:3, 0:3], b) + a[0:3, 3]\
                   for a, b in zip(affines, centers)])

# test 4: crash test compose method
A = Affine(random_affine())
TA = T.compose(A)

# test 5: crash test left compose method
AT = A.compose(T)

z = AT.apply(xyz)
za = A.compose(Affine(raf)).apply(xyz)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from ..image_list import ImageList, iter_axis
from ..image import Image
from ....io.api import load_image
from ....core.reference.coordinate_map import (AxisError, CoordinateSystem,
                                               AffineTransform)

from ....testing import (funcfile, assert_true, assert_equal, assert_raises,
                        assert_almost_equal)


FIMG = load_image(funcfile)


def test_il_init():
    images = list(iter_axis(FIMG, 't'))
    imglst = ImageList(images)
    assert_equal(len(imglst), 20)
    element = imglst[1]
    assert_equal(element.shape, (17, 21, 3))
    assert_equal(element.coordmap, FIMG[...,1].coordmap)
    # Test bad construction
    bad_images = images + [np.zeros((17, 21, 3))]
    assert_raises(ValueError, ImageList, bad_images)
    a = np.arange(10)
    assert_raises(ValueError, ImageList, a)
    # Test empty ImageList
    emplst = ImageList()
    assert_equal(len(emplst.list), 0)


def test_il_from_image():
    exp_shape = (17, 21, 3, 20)
    assert_equal(FIMG.shape, exp_shape)
    # from_image construction
    imglst = ImageList.from_image(FIMG, axis=-1)
    # Test axis must be specified
    assert_raises(ValueError, ImageList.from_image, FIMG)
    assert_raises(ValueError, ImageList.from_image, FIMG, None)
    # check all the axes
    for i in range(4):
        order = range(4)
        order.remove(i)
        order.insert(0,i)
        img_re_i = FIMG.reordered_reference(order).reordered_axes(order)
        imglst_i = ImageList.from_image(FIMG, axis=i)
        assert_equal(imglst_i.list[0].shape, img_re_i.shape[1:])
        # check the affine as well
        assert_almost_equal(imglst_i.list[0].affine, img_re_i.affine[1:,1:])
    # length of image list should match number of frames
    assert_equal(len(imglst), FIMG.shape[3])
    # check the affine
    A = np.identity(4)
    A[:3,:3] = FIMG.affine[:3,:3]
    A[:3,-1] = FIMG.affine[:3,-1]
    assert_almost_equal(imglst.list[0].affine, A)
    # Check other ways of naming axis
    assert_equal(len(ImageList.from_image(FIMG, axis='t')), 20)
    # Input and output axis names work
    new_cmap = AffineTransform(CoordinateSystem('ijkl'),
                               FIMG.coordmap.function_range,
                               FIMG.coordmap.affine)
    fimg2 = Image(FIMG.get_data(), new_cmap)
    assert_equal(len(ImageList.from_image(fimg2, axis='t')), 20)
    assert_equal(len(ImageList.from_image(fimg2, axis='l')), 20)
    assert_raises(AxisError, ImageList.from_image, FIMG, 'q')
    # Check non-dropping case
    ndlist = ImageList.from_image(FIMG, axis='t', dropout=False)
    element = ndlist[1]
    assert_equal(element.coordmap, FIMG[...,1].coordmap)


def test_il_slicing_dicing():
    imglst = ImageList.from_image(FIMG, -1)
    # Slicing an ImageList should return an ImageList
    sublist = imglst[2:5]
    assert_true(isinstance(sublist, ImageList))
    # Except when we're indexing one element
    assert_true(isinstance(imglst[0], Image))
    # Verify array interface
    # test __array__
    assert_true(isinstance(sublist.get_list_data(axis=0), np.ndarray))
    # Test __setitem__
    sublist[2] = sublist[0]
    assert_equal(sublist[0].get_data().mean(),
                 sublist[2].get_data().mean())
    # Test iterator
    for x in sublist:
        assert_true(isinstance(x, Image))
        assert_equal(x.shape, FIMG.shape[:3])

    # Test image_list.get_list_data(axis = an_axis)
    funcim = load_image(funcfile)
    ilist = ImageList.from_image(funcim, axis='t')

    # make sure that we pass an axis
    assert_raises(ValueError, ImageList.get_list_data, ilist, None)
    assert_raises(ValueError, ImageList.get_list_data, ilist)

    # make sure that axis that don't exist makes the function fail
    assert_raises(ValueError, ImageList.get_list_data, ilist, 4)
    assert_raises(ValueError, ImageList.get_list_data, ilist, -5)

    # make sure that axis is put in the right place in the result array
    # image of ilist have dimension (17,21,3), lenght(ilist) = 20.
    data = ilist.get_list_data(axis=0)
    assert_equal(data.shape, (20, 17, 21, 3))

    data = ilist.get_list_data(axis=1)
    assert_equal(data.shape, (17, 20, 21, 3))

    data = ilist.get_list_data(axis=2)
    assert_equal(data.shape, (17, 21, 20, 3))

    data = ilist.get_list_data(axis=3)
    assert_equal(data.shape, (17, 21, 3, 20))

    data = ilist.get_list_data(axis=-1)
    assert_equal(data.shape, (17, 21, 3, 20))

    data = ilist.get_list_data(axis=-2)
    assert_equal(data.shape, (17, 21, 20, 3))



""" Testing brain segmentation module
"""

import numpy as np

from nose.tools import assert_equal, assert_almost_equal
from numpy.testing import assert_array_almost_equal

from ..segmentation import Segmentation
from ..brain_segmentation import BrainT1Segmentation

from ....io.files import load as load_image
from ....testing import anatfile

anat_img = load_image(anatfile)
anat_mask = anat_img.get_data() > 0

DIMS = (30, 30, 20)


def _check_dims(x, ndim, shape):
    if isinstance(shape, int):
        shape = (shape, )
    for i in range(ndim):
        assert_equal(x.shape[i], shape[i])


def _test_brain_seg(model, niters=3, beta=0, ngb_size=6, init_params=None,
                    convert=True):
    S = BrainT1Segmentation(anat_img.get_data(), mask=anat_mask,
                            model=model, niters=niters, beta=beta,
                            ngb_size=ngb_size, init_params=init_params,
                            convert=convert)
    shape = anat_img.shape
    if convert:
        nclasses = 3
    else:
        nclasses = S.mixmat.shape[0]
    # Check that the class attributes have appropriate dimensions
    _check_dims(S.ppm, 4, list(shape) + [nclasses])
    _check_dims(S.label, 3, shape)
    _check_dims(S.mu, 1, S.mixmat.shape[0])
    _check_dims(S.sigma, 1, S.mixmat.shape[0])
    # Check that probabilities are zero outside the mask and sum up to
    # one inside the mask
    assert_almost_equal(S.ppm[True - S.mask].sum(-1).max(), 0)
    assert_almost_equal(S.ppm[S.mask].sum(-1).min(), 1)
    # Check that labels are zero outside the mask and > 1 inside the
    # mask
    assert_almost_equal(S.label[True - S.mask].max(), 0)
    assert_almost_equal(S.label[S.mask].min(), 1)


def test_brain_seg1():
    _test_brain_seg('3k', niters=3, beta=0.0, ngb_size=6)


def test_brain_seg2():
    _test_brain_seg('3k', niters=3, beta=0.5, ngb_size=6)


def test_brain_seg3():
    _test_brain_seg('4k', niters=3, beta=0.5, ngb_size=6)


def test_brain_seg4():
    _test_brain_seg('4k', niters=3, beta=0.5, ngb_size=26)


def test_brain_seg5():
    _test_brain_seg(np.array([[1., 0., 0.],
                              [1., 0., 0.],
                              [0., 1., 0.],
                              [0., 1., 0.],
                              [0., 0., 1.]]),
                    niters=3, beta=0.5, ngb_size=6)


def test_brain_seg6():
    _test_brain_seg('3k', niters=3, beta=0.5, ngb_size=6,
                    convert=False)


def test_brain_seg7():
    mu = np.array([0, 50, 100])
    sigma = np.array([1000, 2000, 3000])
    _test_brain_seg('3k', niters=3, beta=0.5, ngb_size=6,
                    init_params=(mu, sigma))


def _test_segmentation(S, nchannels=1):
    assert_equal(S.nchannels, nchannels)
    nef = S.normalized_external_field()
    assert_array_almost_equal(nef.sum(-1), np.ones(nef.shape[0]))
    S.run(niters=5)
    label = S.map()
    assert_equal(label.ndim, 3)
    assert_equal(label.dtype, 'uint8')
    assert isinstance(S.free_energy(), float)


def test_segmentation_3d():
    data = np.random.random(DIMS)
    _test_segmentation(Segmentation(data, mu=[0.25, 0.75], sigma=[1, 1]))


def test_segmentation_3d_with_MRF():
    data = np.random.random(DIMS)
    _test_segmentation(Segmentation(data, mu=[0.25, 0.75],
                                    sigma=[1, 1], beta=.2))


def test_segmentation_3d_with_mask():
    data = np.random.random(DIMS)
    mask = data > .1
    if mask[0].size < 1:
        return
    _test_segmentation(Segmentation(data, mu=[0.25, 0.75],
                                    sigma=[1, 1], mask=mask))


def test_segmentation_3d_multichannel():
    data = np.random.random(list(DIMS) + [2])
    mask = data[..., 0] > .1
    if mask[0].size < 1:
        return
    _test_segmentation(Segmentation(data,
                                    mu=[[0.25, 0.25], [0.75, 0.75]],
                                    sigma=[np.eye(2), np.eye(2)],
                                    mask=mask),
                       nchannels=2)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The base volumetric field interface 

This defines the nipy volumetric structure interface.
"""
from ..transforms.transform import CompositionError

################################################################################
# class `VolumeField`
################################################################################

class VolumeField(object):
    """ The base volumetric structure.

        This object represents numerical values embedded in a
        3-dimensional world space (called a field in physics and
        engineering)
        
        This is an abstract base class: it defines the interface, but not
        the logics.

        Attributes
        ----------

        world_space: string 
            World space the data is embedded in. For instance `mni152`.
        metadata: dictionnary
            Optional, user-defined, dictionnary used to carry around
            extra information about the data as it goes through
            transformations. The consistency of this information is not
            maintained as the data is modified.

    """
    #---------------------------------------------------------------------------
    # Public attributes -- VolumeField interface
    #---------------------------------------------------------------------------

    # The name of the reference coordinate system
    world_space = ''

    # User defined meta data
    metadata = dict()

    #---------------------------------------------------------------------------
    # Public methods -- VolumeField interface
    #---------------------------------------------------------------------------

    def get_transform(self):
        """ Returns the transform object associated with the volumetric
            structure which is a general description of the mapping from 
            the values to the world space.
            
            Returns
            -------
            transform : nipy.datasets.Transform object
        """
        raise NotImplementedError 


    def resampled_to_img(self, target_image, interpolation=None):
        """ Resample the volume to be sampled similarly than the target 
            volumetric structure.

            Parameters
            ----------
            target_image : nipy volume 
                Nipy volume structure onto the grid of which the data will be
                resampled.
            interpolation : None, 'continuous' or 'nearest', optional
                Interpolation type used when calculating values in
                different word spaces. If None, the volume's interpolation
                logic is used.

            Returns
            -------
            resampled_image : nipy_image
                New nipy image with the data resampled.

            Notes
            -----
            Both the target image and the original image should be
            embedded in the same world space.
        """
        # IMPORTANT: Polymorphism can be implemented by walking the 
        # MRO and finding a method that does not raise
        # NotImplementedError. 
        raise NotImplementedError


    def as_volume_img(self, affine=None, shape=None,
                        interpolation=None, copy=True):
        """ Resample the image to be an image with the data points lying
            on a regular grid with an affine mapping to the word space (a
            nipy VolumeImg).

            Parameters
            ----------
            affine: 4x4 or 3x3 ndarray, optional
                Affine of the new voxel grid or transform object pointing
                to the new voxel coordinate grid. If a 3x3 ndarray is given, 
                it is considered to be the rotation part of the affine, 
                and the best possible bounding box is calculated,
                in this case, the shape argument is not used. If None
                is given, a default affine is provided by the image.
            shape: (n_x, n_y, n_z), tuple of integers, optional
                The shape of the grid used for sampling, if None
                is given, a default affine is provided by the image.
            interpolation : None, 'continuous' or 'nearest', optional
                Interpolation type used when calculating values in
                different word spaces. If None, the image's interpolation
                logic is used.

            Returns
            -------
            resampled_image : nipy VolumeImg
                New nipy VolumeImg with the data sampled on the grid
                defined by the affine and shape.

            Notes
            -----
            The coordinate system of the image is not changed: the
            returned image points to the same world space.
        """
        raise NotImplementedError


    def values_in_world(self, x, y, z, interpolation=None):
        """ Return the values of the data at the world-space positions given by 
            x, y, z

            Parameters
            ----------
            x : number or ndarray
                x positions in world space, in other words milimeters
            y : number or ndarray
                y positions in world space, in other words milimeters.
                The shape of y should match the shape of x
            z : number or ndarray
                z positions in world space, in other words milimeters.
                The shape of z should match the shape of x
            interpolation : None, 'continuous' or 'nearest', optional
                Interpolation type used when calculating values in
                different word spaces. If None, the image's interpolation
                logic is used.

            Returns
            -------
            values : number or ndarray
                Data values interpolated at the given world position.
                This is a number or an ndarray, depending on the shape of
                the input coordinate.
        """
        raise NotImplementedError


    def composed_with_transform(self, w2w_transform):
        """ Return a new image embedding the same data in a different 
            word space using the given world to world transform.

            Parameters
            ----------
            w2w_transform : transform object
                The transform object giving the mapping between
                the current world space of the image, and the new
                word space.

            Returns
            --------
            remapped_image : nipy image
                An image containing the same data, expressed
                in the new world space.
        """
        if not w2w_transform.input_space == self.world_space:
            raise CompositionError(
                "The transform given does not apply to "
                "the image's world space:\n%s\n\n%s" % 
                (w2w_transform, self)
                )
        new_img = self._apply_transform(w2w_transform)
        new_img.world_space = w2w_transform.output_space
        return new_img 


    #---------------------------------------------------------------------------
    # Private methods
    #---------------------------------------------------------------------------

    # The subclasses should implement __repr__, __copy__, __deepcopy__,
    # __eq__ 
    # TODO: We need to implement (or check if implemented) hashing,
    # weakref, pickling? 
        
    def _apply_transform(self, w2w_transform):
        """ Implement this method to put in the logic of applying a
            transformation on the image class.
        """
        raise NotImplementedError



""" Tests for coordinate_system module
"""
import numpy as np

from ..coordinate_system import (CoordinateSystem, CoordinateSystemError,
                                 is_coordsys, is_coordsys_maker, product,
                                 safe_dtype, CoordSysMaker, CoordSysMakerError)

from nose.tools import (assert_true, assert_false, assert_equal, assert_raises,
                        assert_not_equal)


class empty(object):
    pass

E = empty()

def setup():
    E.name = "test"
    E.axes = ('i', 'j', 'k')
    E.coord_dtype = np.float32
    E.cs = CoordinateSystem(E.axes, E.name, E.coord_dtype)


def test_CoordinateSystem():
    assert_equal(E.cs.name, E.name)
    assert_equal(E.cs.coord_names, E.axes)
    assert_equal(E.cs.coord_dtype, E.coord_dtype)


def test_iterator_coordinate():
    def gen():
        yield 'i'
        yield 'j'
        yield 'k'
    coordsys = CoordinateSystem(gen(), name='test_iter')
    assert_equal(coordsys.coord_names, ('i','j','k'))


def test_ndim():
    cs = CoordinateSystem('xy')
    assert_equal(cs.ndim, 2)
    cs = CoordinateSystem('ijk')
    assert_equal(cs.ndim, 3)


def test_unique_coord_names():
    unique = ('i','j','k')
    notuniq = ('i','i','k')
    coordsys = CoordinateSystem(unique)
    assert_equal(coordsys.coord_names, unique)
    assert_raises(ValueError, CoordinateSystem, notuniq)


def test_dtypes():
    # invalid dtypes
    dtypes = np.sctypes['others']
    dtypes.remove(np.object)
    for dt in dtypes:
        assert_raises(ValueError, CoordinateSystem, 'ijk', 'test', dt)
    # compound dtype
    dtype = np.dtype([('field1', '<f8'), ('field2', '<i4')])
    assert_raises(ValueError, CoordinateSystem, 'ijk', 'test', dtype)
    # valid dtypes
    dtypes = (np.sctypes['int'] + np.sctypes['float'] + np.sctypes['complex'] +
              [np.object])
    for dt in dtypes:
        cs = CoordinateSystem('ij', coord_dtype=dt)
        assert_equal(cs.coord_dtype, dt)
        cs_dt = [(f, dt) for f in 'ij']
        assert_equal(cs.dtype, np.dtype(cs_dt))
        # Check product too
        cs2 = CoordinateSystem('xy', coord_dtype=dt)
        assert_equal(product(cs, cs2),
                     CoordinateSystem('ijxy', name='product', coord_dtype=dt))
    # verify assignment fails
    assert_raises(AttributeError, setattr, cs, 'dtype', np.dtype(cs_dt))
    assert_raises(AttributeError, setattr, cs, 'coord_dtype', np.float)


def test_readonly_attrs():
    cs = E.cs
    assert_raises(AttributeError, setattr, cs, 'coord_dtype',
                  np.dtype(np.int32))
    assert_raises(AttributeError, setattr, cs, 'coord_names',
                  ['a','b','c'])
    assert_raises(AttributeError, setattr, cs, 'dtype',
                  np.dtype([('i', '<f4'), ('j', '<f4'), ('k', '<f4')]))
    assert_raises(AttributeError, setattr, cs, 'ndim', 4)


def test_index():
    cs = CoordinateSystem('ijk')
    assert_equal(cs.index('i'), 0)
    assert_equal(cs.index('j'), 1)
    assert_equal(cs.index('k'), 2)
    assert_raises(ValueError, cs.index, 'x')


def test__ne__():
    cs1 = CoordinateSystem('ijk')
    cs2 = CoordinateSystem('xyz')
    assert_true(cs1 != cs2)
    cs1 = CoordinateSystem('ijk', coord_dtype='float')
    cs2 = CoordinateSystem('ijk', coord_dtype='int')
    assert_true(cs1 != cs2)


def test___eq__():
    c0 = CoordinateSystem('ijk', 'my name', np.float32)
    c1 = CoordinateSystem('ijk', 'my name', np.float32)
    assert_equal(c0, c1)
    c2 = CoordinateSystem('ijk', 'another name', np.float32)
    assert_not_equal(c0, c2)
    c3 = CoordinateSystem('ijq', 'my name', np.float32)
    assert_not_equal(c0, c3)
    c4 = CoordinateSystem('ijk', 'my name', np.float64)
    assert_not_equal(c0, c4)


def test_similar_to():
    c0 = CoordinateSystem('ijk', 'my name', np.float32)
    c1 = CoordinateSystem('ijk', 'my name', np.float32)
    assert_true(c0.similar_to(c1))
    c2 = CoordinateSystem('ijk', 'another name', np.float32)
    assert_true(c0.similar_to(c2))
    c3 = CoordinateSystem('ijq', 'my name', np.float32)
    assert_false(c0.similar_to(c3))
    c4 = CoordinateSystem('ijk', 'my name', np.float64)
    assert_false(c0.similar_to(c4))


def test___str__():
    s = str(E.cs)
    assert_equal(s, "CoordinateSystem(coord_names=('i', 'j', 'k'), name='test', coord_dtype=float32)")


def test_is_coordsys():
    # Test coordinate system check
    csys = CoordinateSystem('ijk')
    assert_true(is_coordsys(csys))
    class C(object): pass
    c = C()
    assert_false(is_coordsys(c))
    c.coord_names = []
    assert_false(is_coordsys(c))
    c.name = ''
    assert_false(is_coordsys(c))
    c.coord_dtype = np.float
    assert_true(is_coordsys(c))
    # Distinguish from CoordSysMaker
    class C(object):
        coord_names = []
        name = ''
        coord_dtype=np.float
        def __call__(self):
            pass
    assert_false(is_coordsys(C()))
    assert_false(is_coordsys(CoordSysMaker('xyz')))


def test_checked_values():
    cs = CoordinateSystem('ijk', name='voxels', coord_dtype=np.float32)
    x = np.array([1, 2, 3], dtype=np.int16)
    xc = cs._checked_values(x)
    np.allclose(xc, x)
    # wrong shape
    assert_raises(CoordinateSystemError, cs._checked_values, x.reshape(3,1))
    # wrong length
    assert_raises(CoordinateSystemError, cs._checked_values, x[0:2])
    # wrong dtype
    x = np.array([1,2,3], dtype=np.float64)
    assert_raises(CoordinateSystemError, cs._checked_values, x)


def test_safe_dtype():
    assert_raises(TypeError, safe_dtype, type('foo'))
    assert_raises(TypeError, safe_dtype, type('foo'), np.float64)
    assert_raises(TypeError, safe_dtype, [('x', 'f8')])
    valid_dtypes = []
    valid_dtypes.extend(np.sctypes['complex'])
    valid_dtypes.extend(np.sctypes['float'])
    valid_dtypes.extend(np.sctypes['int'])
    valid_dtypes.extend(np.sctypes['uint'])
    for dt in valid_dtypes:
        sdt = safe_dtype(dt)
        assert_equal(sdt, dt)
    # test a few upcastings
    dt = safe_dtype(np.float32, np.int16, np.bool)
    assert_equal(dt, np.float32)
    dt = safe_dtype(np.float32, np.int64, np.uint32)
    assert_equal(dt, np.float64)
    # test byteswapped types - isbuiltin will be false
    orig_dt = np.dtype('f')
    dt = safe_dtype(orig_dt, np.int64, np.uint32)
    assert_equal(dt, np.float64)
    swapped_dt = orig_dt.newbyteorder()
    dt = safe_dtype(swapped_dt, np.int64, np.uint32)
    assert_equal(dt, np.float64)


def test_product():
    # int32 + int64 => int64
    ax1 = CoordinateSystem('x', coord_dtype=np.int32)
    ax2 = CoordinateSystem('y', coord_dtype=np.int64)
    cs = product(ax1, ax2)
    # assert up-casting of dtype
    assert_equal(cs.coord_dtype, np.dtype(np.int64))
    # assert composed dtype 
    assert_equal(cs.dtype, np.dtype([('x', np.int64), ('y', np.int64)]))
    # the axes should be typecast in the CoordinateSystem but
    # uneffected themselves
    assert_equal(ax1.dtype, np.dtype([('x', np.int32)]))
    assert_equal(ax2.dtype, np.dtype([('y', np.int64)]))

    # float32 + int64 => float64
    ax1 = CoordinateSystem('x', coord_dtype=np.float32)
    cs = product(ax1, ax2)
    assert_equal(cs.coord_dtype, np.dtype(np.float64))
    assert_equal(cs.dtype, np.dtype([('x', np.float64),
                                     ('y', np.float64)]))
    # int16 + complex64 => complex64
    ax1 = CoordinateSystem('x', coord_dtype=np.int16)
    ax2 = CoordinateSystem('y', coord_dtype=np.complex64)
    # Order of the params effects order of dtype but not resulting value type
    cs = product(ax2, ax1)
    assert_equal(cs.coord_dtype, np.complex64)
    assert_equal(cs.dtype, np.dtype([('y', np.complex64),
                                     ('x', np.complex64)]))
    # Passing name as argument
    cs = product(ax2, ax1, name='a name')
    assert_equal(cs.name, 'a name')
    # Anything else as kwarg -> error
    assert_raises(TypeError, product, ax2, ax1, newarg='a name')


def test_coordsys_maker():
    # Things that help making coordinate maps
    ax_names = list('ijklm')
    nl = len(ax_names)
    cs_maker = CoordSysMaker(ax_names, 'myname')
    for i in range(1,nl+1):
        assert_equal(cs_maker(i),
                     CoordinateSystem(ax_names[:i], 'myname', np.float))
    assert_raises(CoordSysMakerError, cs_maker, nl+1)
    # You can pass in your own name
    assert_equal(cs_maker(i, 'anothername'),
                 CoordinateSystem(ax_names[:i+1], 'anothername', np.float))
    # And your own dtype if you really want
    assert_equal(cs_maker(i, coord_dtype=np.int32),
                 CoordinateSystem(ax_names[:i+1], 'myname', np.int32))


def test_is_coordsys_maker():
    # Test coordinate system check
    cm = CoordSysMaker('xyz')
    assert_true(is_coordsys_maker(cm))
    class C(object): pass
    c = C()
    assert_false(is_coordsys_maker(c))
    c.coord_names = []
    assert_false(is_coordsys_maker(c))
    c.name = ''
    assert_false(is_coordsys_maker(c))
    c.coord_dtype = np.float
    assert_false(is_coordsys_maker(c))
    # Distinguish from CoordinateSystem
    class C(object):
        coord_names = []
        name = ''
        coord_dtype=np.float
        def __call__(self):
            pass
    assert_true(is_coordsys_maker(C()))
    assert_false(is_coordsys_maker(CoordinateSystem('ijk')))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This test ensures that the design matrices of the FIAC dataset match
with fMRIstat's, at least on one block and one event trial.

Taylor, J.E. & Worsley, K.J. (2005). \'Inference for 
    magnitudes and delays of responses in the FIAC data using 
    BRAINSTAT/FMRISTAT\'. Human Brain Mapping, 27,434-441
"""

import numpy as np

from ... import utils, hrf, design
from .. import hrf as delay

from nipy.algorithms.statistics.models.regression import OLSModel
from nipy.algorithms.statistics.formula import formulae

from nipy.utils.compat3 import to_str

# testing imports
from nipy.testing import (dec, assert_true, assert_almost_equal)

# Local imports
from .FIACdesigns import (descriptions, fmristat, altdescr,
                          N_ROWS, time_vector)

t = formulae.make_recarray(time_vector, 't')


def protocol(recarr, design_type, *hrfs):
    """ Create an object that can evaluate the FIAC

    Subclass of formulae.Formula, but not necessary.

    Parameters
    ----------
    recarr : (N,) structured array
       with fields 'time' and 'event'
    design_type : str
       one of ['event', 'block'].  Handles how the 'begin' term is
       handled.  For 'block', the first event of each block is put in
       this group. For the 'event', only the first event is put in this
       group. The 'begin' events are convolved with hrf.glover.
    hrfs: symoblic HRFs
       Each event type ('SSt_SSp','SSt_DSp','DSt_SSp','DSt_DSp') is
       convolved with each of these HRFs in order.

    Returns
    -------
    f: Formula
       Formula for constructing design matrices.
    contrasts : dict
       Dictionary of the contrasts of the experiment.
    """
    event_types = np.unique(recarr['event'])
    N = recarr.size
    if design_type == 'block':
        keep = np.not_equal((np.arange(N)) % 6, 0)
    else:
        keep = np.greater(np.arange(N), 0)
    # This first frame was used to model out a potentially
    # 'bad' first frame....
    _begin = recarr['time'][~keep]

    termdict = {}
    termdict['begin'] = utils.define('begin', utils.events(_begin, f=hrf.glover))
    drift = formulae.natural_spline(utils.T,
                                   knots=[N_ROWS/2.+1.25],
                                   intercept=True)
    for i, t in enumerate(drift.terms):
        termdict['drift%d' % i] = t
    # After removing the first frame, keep the remaining
    # events and times
    times = recarr['time'][keep]
    events = recarr['event'][keep]

    # Now, specify the experimental conditions.  This creates expressions named
    # SSt_SSp0, SSt_SSp1, etc.  with one expression for each (eventtype, hrf)
    # pair
    for v in event_types:
        k = np.array([events[i] == v for i in range(times.shape[0])])
        for l, h in enumerate(hrfs):
            # Make sure event type is a string (not byte string)
            term_name = '%s%d' % (to_str(v), l)
            termdict[term_name] = utils.define(term_name,
                                               utils.events(times[k], f=h))
    f = formulae.Formula(termdict.values())
    Tcontrasts = {}
    Tcontrasts['average'] = (termdict['SSt_SSp0'] + termdict['SSt_DSp0'] +
                             termdict['DSt_SSp0'] + termdict['DSt_DSp0']) / 4.
    Tcontrasts['speaker'] = (termdict['SSt_DSp0'] - termdict['SSt_SSp0'] +
                             termdict['DSt_DSp0'] - termdict['DSt_SSp0']) * 0.5
    Tcontrasts['sentence'] = (termdict['DSt_DSp0'] + termdict['DSt_SSp0'] -
                              termdict['SSt_DSp0'] - termdict['SSt_SSp0']) * 0.5
    Tcontrasts['interaction'] = (termdict['SSt_SSp0'] - termdict['SSt_DSp0'] -
                                 termdict['DSt_SSp0'] + termdict['DSt_DSp0'])
    # Ftest
    Fcontrasts = {}
    Fcontrasts['overall1'] = formulae.Formula(Tcontrasts.values())

    return f, Tcontrasts, Fcontrasts


def altprotocol(d, design_type, *hrfs):
    """ Create an object that can evaluate the FIAC.

    Subclass of formulae.Formula, but not necessary.

    Parameters
    ----------
    d : np.recarray
       recarray defining design in terms of time, sentence speaker

    design_type : str in ['event', 'block']
        Handles how the 'begin' term is handled.
        For 'block', the first event of each block
        is put in this group. For the 'event', 
        only the first event is put in this group.

        The 'begin' events are convolved with hrf.glover.

    hrfs: symoblic HRFs
        Each event type ('SSt_SSp','SSt_DSp','DSt_SSp','DSt_DSp')
        is convolved with each of these HRFs in order.

    """
    if design_type == 'block':
        keep = np.not_equal((np.arange(d.time.shape[0])) % 6, 0)
    else:
        keep = np.greater(np.arange(d.time.shape[0]), 0)

    # This first frame was used to model out a potentially
    # 'bad' first frame....

    _begin = d.time[~keep]
    d = d[keep]

    termdict = {}
    termdict['begin'] = utils.define('begin', utils.events(_begin, f=hrf.glover))
    drift = formulae.natural_spline(utils.T,
                                   knots=[N_ROWS/2.+1.25],
                                   intercept=True)
    for i, t in enumerate(drift.terms):
        termdict['drift%d' % i] = t

    # Now, specify the experimental conditions
    # The elements of termdict are DiracDeltas, rather than HRFs

    st = formulae.Factor('sentence', ['DSt', 'SSt'])
    sp = formulae.Factor('speaker', ['DSp', 'SSp'])

    indic = {}
    indic['sentence'] =  st.main_effect
    indic['speaker'] =  sp.main_effect
    indic['interaction'] = st.main_effect * sp.main_effect
    indic['average'] = formulae.I

    for key in indic.keys():
        # The matrix signs will be populated with +- 1's
        # d is the recarray having fields ('time', 'sentence', 'speaker')
        signs = indic[key].design(d, return_float=True)

        for l, h in enumerate(hrfs):

            # symb is a sympy expression representing a sum
            # of [h(t-_t) for _t in d.time]
            symb = utils.events(d.time, amplitudes=signs, f=h)

            # the values of termdict will have keys like
            # 'average0', 'speaker1'
            # and values  that are sympy expressions like average0(t), 
            # speaker1(t)
            termdict['%s%d' % (key, l)] = utils.define("%s%d" % (key, l), symb)

    f = formulae.Formula(termdict.values())

    Tcontrasts = {}
    Tcontrasts['average'] = termdict['average0']
    Tcontrasts['speaker'] = termdict['speaker0']
    Tcontrasts['sentence'] = termdict['sentence0']
    Tcontrasts['interaction'] = termdict['interaction0']

    # F tests

    Fcontrasts = {}
    Fcontrasts['overall1'] = formulae.Formula(Tcontrasts.values())

    nhrf = len(hrfs)
    Fcontrasts['averageF'] = formulae.Formula([termdict['average%d' % j] for j in range(nhrf)])
    Fcontrasts['speakerF'] = formulae.Formula([termdict['speaker%d' % j] for j in range(nhrf)])
    Fcontrasts['sentenceF'] = formulae.Formula([termdict['sentence%d' % j] for j in range(nhrf)])
    Fcontrasts['interactionF'] = formulae.Formula([termdict['interaction%d' % j] for j in range(nhrf)])

    Fcontrasts['overall2'] = Fcontrasts['averageF'] + Fcontrasts['speakerF'] + Fcontrasts['sentenceF'] + Fcontrasts['interactionF']

    return f, Tcontrasts, Fcontrasts


def create_protocols():
    # block and event protocols
    block, bTcons, bFcons = protocol(descriptions['block'], 'block', *delay.spectral)
    event, eTcons, eFcons = protocol(descriptions['event'], 'event', *delay.spectral)

    # Now create the design matrices and contrasts
    # The 0 indicates that it will be these columns
    # convolved with the first HRF
    X = {}
    c = {}
    D = {}
    for f, cons, design_type in [(block, bTcons, 'block'), (event, eTcons, 'event')]:
        X[design_type], c[design_type] = f.design(t, contrasts=cons)
        D[design_type] = f.design(t, return_float=False)
    return X, c, D


def test_altprotocol():
    block, bT, bF = protocol(descriptions['block'], 'block', *delay.spectral)
    event, eT, eF = protocol(descriptions['event'], 'event', *delay.spectral)

    blocka, baT, baF = altprotocol(altdescr['block'], 'block', *delay.spectral)
    eventa, eaT, eaF = altprotocol(altdescr['event'], 'event', *delay.spectral)

    for c in bT.keys():
        baf = baT[c]
        if not isinstance(baf, formulae.Formula):
            baf = formulae.Formula([baf])

        bf = bT[c]
        if not isinstance(bf, formulae.Formula):
            bf = formulae.Formula([bf])

    X = baf.design(t, return_float=True)
    Y = bf.design(t, return_float=True)
    if X.ndim == 1:
        X.shape = (X.shape[0], 1)
    m = OLSModel(X)
    r = m.fit(Y)
    remaining = (r.resid**2).sum() / (Y**2).sum()
    assert_almost_equal(remaining, 0)

    for c in bF.keys():
        baf = baF[c]
        if not isinstance(baf, formulae.Formula):
            baf = formulae.Formula([baf])

        bf = bF[c]
        if not isinstance(bf, formulae.Formula):
            bf = formulae.Formula([bf])

    X = baf.design(t, return_float=True)
    Y = bf.design(t, return_float=True)
    if X.ndim == 1:
        X.shape = (X.shape[0], 1)
    m = OLSModel(X)
    r = m.fit(Y)
    remaining = (r.resid**2).sum() / (Y**2).sum()
    assert_almost_equal(remaining, 0)


def matchcol(col, X):
    """ Find the row in X with the highest correlation with 1D col.

    Used to find matching columns in fMRIstat's design with the design
    created by Protocol. Not meant as a generic helper function.
    """
    c = np.array([np.corrcoef(col, X[i])[0,1] for i in range(X.shape[0])])
    c = np.nan_to_num(c)
    ind = np.argmax(np.abs(c))
    return ind, c[ind]


def test_agreement():
    # The test: does Protocol manage to recreate the design of fMRIstat?
    X, c, D = create_protocols()
    for design_type in ['event', 'block']:
        dd = D[design_type]
        for i in range(X[design_type].shape[1]):
            _, cmax = matchcol(X[design_type][:,i], fmristat[design_type])
            if not dd.dtype.names[i].startswith('ns'):
                assert_true(np.greater(np.abs(cmax), 0.999))


@dec.slow
def test_event_design():
    block = altdescr['block']
    event = altdescr['event']
    t = time_vector

    bkeep = np.not_equal((np.arange(block.time.shape[0])) % 6, 0)
    ekeep = np.greater(np.arange(event.time.shape[0]), 0)

    # Even though there is a FIAC block experiment
    # the design is represented as an event design
    # with the same event repeated several times in a row...

    Xblock, cblock = design.event_design(block[bkeep], t, hrfs=delay.spectral)
    Xevent, cevent = design.event_design(event[ekeep], t, hrfs=delay.spectral)



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = \
"""
Example of a script that crates a 'hierarchical roi' structure from the blob
model of an image

Needs matplotlib

Author: Bertrand Thirion, 2008-2009
"""
print(__doc__)

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

import nipy.labs.spatial_models.hroi as hroi
import nipy.labs.utils.simul_multisubject_fmri_dataset as simul
from nipy.labs.spatial_models.discrete_domain import domain_from_binary_array

##############################################################################
# simulate the data
shape = (60, 60)
pos = np.array([[12, 14], [20, 20], [30, 20]])
ampli = np.array([3, 4, 4])

dataset = simul.surrogate_2d_dataset(n_subj=1, shape=shape, pos=pos,
                                     ampli=ampli, width=10.0).squeeze()

# create a domain descriptor associated with this
domain = domain_from_binary_array(dataset ** 2 > 0)

nroi = hroi.HROI_as_discrete_domain_blobs(domain, dataset.ravel(),
                                          threshold=2., smin=5)

n1 = nroi.copy()
nroi.reduce_to_leaves()

td = n1.make_forest().depth_from_leaves()
root = np.argmax(td)
lv = n1.make_forest().get_descendants(root)
u = nroi.make_graph().cc()

flat_data = dataset.ravel()
activation = [flat_data[nroi.select_id(id, roi=False)]
              for id in nroi.get_id()]
nroi.set_feature('activation', activation)

label = np.reshape(n1.label, shape)
label_ = np.reshape(nroi.label, shape)

# make a figure
plt.figure(figsize=(10, 4))
plt.subplot(1, 3, 1)
plt.imshow(np.squeeze(dataset))
plt.title('Input map')
plt.axis('off')
plt.subplot(1, 3, 2)
plt.title('Nested Rois')
plt.imshow(label, interpolation='Nearest')
plt.axis('off')
plt.subplot(1, 3, 3)
plt.title('Leave Rois')
plt.imshow(label_, interpolation='Nearest')
plt.axis('off')
plt.show()




#!/usr/bin/env python

#
# Test BLAS 1
#

from numpy.testing import assert_almost_equal
import numpy as np
from .. import (blas_dnrm2, blas_dasum, blas_ddot, 
                blas_daxpy, blas_dscal)
n = 15


def test_dnrm2():
    x = np.random.rand(n)
    assert_almost_equal(np.sqrt(np.sum(x**2)), blas_dnrm2(x))

def test_dasum():
    x = np.random.rand(n)
    assert_almost_equal(np.sum(np.abs(x)), blas_dasum(x))

def test_ddot():
    x = np.random.rand(n)
    y = np.random.rand(n)
    assert_almost_equal(np.dot(x,y), blas_ddot(x, y))

def test_daxpy():
    x = np.random.rand(n)
    y = np.random.rand(n)
    alpha = np.random.rand()
    assert_almost_equal(alpha*x+y, blas_daxpy(alpha, x, y))

def test_dscal():
    x = np.random.rand(n)
    alpha = np.random.rand()
    assert_almost_equal(alpha*x, blas_dscal(alpha, x))


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Misc tools to find activations and cut on maps
"""

# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
# License: BSD

# Standard scientific libraries imports (more specific imports are
# delayed, so that the part module can be used without them).
import numpy as np
from scipy import stats, ndimage

# Local imports
from ..mask import largest_cc
from ..datasets.transforms.affine_utils import get_bounds

################################################################################
# Functions for automatic choice of cuts coordinates
################################################################################

def coord_transform(x, y, z, affine):
    """ Convert the x, y, z coordinates from one image space to another
        space. 
        
        Parameters
        ----------
        x : number or ndarray
            The x coordinates in the input space
        y : number or ndarray
            The y coordinates in the input space
        z : number or ndarray
            The z coordinates in the input space
        affine : 2D 4x4 ndarray
            affine that maps from input to output space.

        Returns
        -------
        x : number or ndarray
            The x coordinates in the output space
        y : number or ndarray
            The y coordinates in the output space
        z : number or ndarray
            The z coordinates in the output space

        Warning: The x, y and z have their Talairach ordering, not 3D
        numy image ordering.
    """
    coords = np.c_[np.atleast_1d(x).flat, 
                   np.atleast_1d(y).flat, 
                   np.atleast_1d(z).flat,
                   np.ones_like(np.atleast_1d(z).flat)].T
    x, y, z, _ = np.dot(affine, coords)
    return x.squeeze(), y.squeeze(), z.squeeze()


def find_cut_coords(map, mask=None, activation_threshold=None):
    """ Find the center of the largest activation connect component.

        Parameters
        -----------
        map : 3D ndarray
            The activation map, as a 3D image.
        mask : 3D ndarray, boolean, optional
            An optional brain mask.
        activation_threshold : float, optional
            The lower threshold to the positive activation. If None, the 
            activation threshold is computed using find_activation.

        Returns
        -------
        x: float
            the x coordinate in voxels.
        y: float
            the y coordinate in voxels.
        z: float
            the z coordinate in voxels.
    """
    # To speed up computations, we work with partial views of the array,
    # and keep track of the offset
    offset = np.zeros(3) 
    # Deal with masked arrays:
    if hasattr(map, 'mask'):
        not_mask = np.logical_not(map.mask)
        if mask is None:
            mask = not_mask
        else:
            mask *= not_mask
        map = np.asarray(map)
    my_map = map.copy()
    if mask is not None:
        slice_x, slice_y, slice_z = ndimage.find_objects(mask)[0]
        my_map = my_map[slice_x, slice_y, slice_z]
        mask = mask[slice_x, slice_y, slice_z]
        my_map *= mask
        offset += [slice_x.start, slice_y.start, slice_z.start]
    # Testing min and max is faster than np.all(my_map == 0)
    if (my_map.max() == 0) and (my_map.min() == 0):
        return .5*np.array(map.shape)
    if activation_threshold is None:
        activation_threshold = stats.scoreatpercentile(
                                    np.abs(my_map[my_map !=0]).ravel(), 80)
    mask = np.abs(my_map) > activation_threshold-1.e-15
    mask = largest_cc(mask)
    slice_x, slice_y, slice_z = ndimage.find_objects(mask)[0]
    my_map = my_map[slice_x, slice_y, slice_z]
    mask = mask[slice_x, slice_y, slice_z]
    my_map *= mask
    offset += [slice_x.start, slice_y.start, slice_z.start]
    # For the second threshold, we use a mean, as it is much faster,
    # althought it is less robust
    second_threshold = np.abs(np.mean(my_map[mask]))
    second_mask = (np.abs(my_map)>second_threshold)
    if second_mask.sum() > 50:
        my_map *= largest_cc(second_mask)
    cut_coords = ndimage.center_of_mass(np.abs(my_map))
    return cut_coords + offset


################################################################################

def get_mask_bounds(mask, affine):
    """ Return the world-space bounds occupied by a mask given an affine.

        Notes
        -----

        The mask should have only one connect component.

        The affine should be diagonal or diagonal-permuted.
    """
    (xmin, xmax), (ymin, ymax), (zmin, zmax) = get_bounds(mask.shape, affine)
    x_slice, y_slice, z_slice = ndimage.find_objects(mask)[0]
    x_width, y_width, z_width = mask.shape
    xmin, xmax = (xmin + x_slice.start*(xmax - xmin)/x_width,
                  xmin + x_slice.stop *(xmax - xmin)/x_width)
    ymin, ymax = (ymin + y_slice.start*(ymax - ymin)/y_width,
                  ymin + y_slice.stop *(ymax - ymin)/y_width)
    zmin, zmax = (zmin + z_slice.start*(zmax - zmin)/z_width,
                  zmin + z_slice.stop *(zmax - zmin)/z_width)

    return xmin, xmax, ymin, ymax, zmin, zmax
 




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Utilities for extracting masks from EPI images and applying them to time
series.
"""

# Major scientific libraries imports
import numpy as np
from scipy import ndimage

# Neuroimaging libraries imports
from nibabel import load, nifti1, save
from nibabel.loadsave import read_img_data


###############################################################################
# Operating on connect component
###############################################################################

def largest_cc(mask):
    """ Return the largest connected component of a 3D mask array.

    Parameters
    -----------
    mask: 3D boolean array
          3D array indicating a mask.

    Returns
    --------
    mask: 3D boolean array
          3D array indicating a mask, with only one connected component.
    """
    # We use asarray to be able to work with masked arrays.
    mask = np.asarray(mask)
    labels, label_nb = ndimage.label(mask)
    if not label_nb:
        raise ValueError('No non-zero values: no connected components')
    if label_nb == 1:
        return mask.astype(np.bool)
    label_count = np.bincount(labels.ravel())
    # discard 0 the 0 label
    label_count[0] = 0
    return labels == label_count.argmax()


def threshold_connect_components(map, threshold, copy=True):
    """ Given a map with some coefficients set to zero, segment the
        connect components with number of voxels smaller than the
        threshold and set them to 0.

    Parameters
    ----------
    map: ndarray,
         The spatial map to segment
    threshold: scalar,
               The minimum number of voxels to keep a cluster.
    copy: bool, optional
          If copy is false, the input array is modified inplace

    Returns
    -------
    map: ndarray,
         the map with connected components removed
    """
    labels, _ = ndimage.label(map)
    weights = np.bincount(labels.ravel())
    if copy:
        map = map.copy()
    for label, weight in enumerate(weights):
        if label == 0:
            continue
        if weight < threshold:
            map[labels == label] = 0
    return map


###############################################################################
# Utilities to calculate masks
###############################################################################

def compute_mask_files(input_filename, output_filename=None,
                        return_mean=False, m=0.2, M=0.9, cc=1,
                        exclude_zeros=False, opening=2):
    """
    Compute a mask file from fMRI nifti file(s)

    Compute and write the mask of an image based on the grey level
    This is based on an heuristic proposed by T.Nichols:
    find the least dense point of the histogram, between fractions
    m and M of the total image histogram.

    In case of failure, it is usually advisable to increase m.

    Parameters
    ----------
    input_filename : string
        nifti filename (4D) or list of filenames (3D).
    output_filename : string or None, optional
        path to save the output nifti image (if not None).
    return_mean : boolean, optional
        if True, and output_filename is None, return the mean image also, as
        a 3D array (2nd return argument).
    m : float, optional
        lower fraction of the histogram to be discarded.
    M: float, optional
        upper fraction of the histogram to be discarded.
    cc: boolean, optional
        if cc is True, only the largest connect component is kept.
    exclude_zeros: boolean, optional
        Consider zeros as missing values for the computation of the
        threshold. This option is useful if the images have been
        resliced with a large padding of zeros.
    opening: int, optional
        Size of the morphological opening performed as post-processing

    Returns
    -------
    mask : 3D boolean array
        The brain mask
    mean_image : 3d ndarray, optional
        The main of all the images used to estimate the mask. Only
        provided if `return_mean` is True.
    """
    if isinstance(input_filename, basestring):
        # One single filename or image
        nim = load(input_filename)  # load the image from the path
        vol_arr = read_img_data(nim, prefer='unscaled')
        header = nim.get_header()
        affine = nim.get_affine()
        if vol_arr.ndim == 4:
            if isinstance(vol_arr, np.memmap):
                # Get rid of memmapping: it is faster.
                mean_volume = np.array(vol_arr, copy=True).mean(axis=-1)
            else:
                mean_volume = vol_arr.mean(axis=-1)
            # Make a copy, to avoid holding a reference on the full array,
            # and thus polluting the memory.
            first_volume = vol_arr[..., 0].copy()
        elif vol_arr.ndim == 3:
            mean_volume = first_volume = vol_arr
        else:
            raise ValueError('Need 4D file for mask')
        del vol_arr
    else:
        # List of filenames
        if len(list(input_filename)) == 0:
            raise ValueError('input_filename should be a non-empty '
                'list of file names')
        # We have several images, we do mean on the fly,
        # to avoid loading all the data in the memory
        # We do not use the unscaled data here?:
        # if the scalefactor is being used to record real
        # differences in intensity over the run this would break
        for index, filename in enumerate(input_filename):
            nim = load(filename)
            if index == 0:
                first_volume = nim.get_data().squeeze()
                mean_volume = first_volume.copy().astype(np.float32)
                header = nim.get_header()
                affine = nim.get_affine()
            else:
                mean_volume += nim.get_data().squeeze()
        mean_volume /= float(len(list(input_filename)))
    del nim
    if np.isnan(mean_volume).any():
        tmp = mean_volume.copy()
        tmp[np.isnan(tmp)] = 0
        mean_volume = tmp

    mask = compute_mask(mean_volume, first_volume, m, M, cc,
                        opening=opening, exclude_zeros=exclude_zeros)

    if output_filename is not None:
        header['descrip'] = 'mask'
        output_image = nifti1.Nifti1Image(mask.astype(np.uint8),
                                            affine=affine,
                                            header=header)
        save(output_image, output_filename)
    if not return_mean:
        return mask
    else:
        return mask, mean_volume


def compute_mask(mean_volume, reference_volume=None, m=0.2, M=0.9,
                        cc=True, opening=2, exclude_zeros=False):
    """
    Compute a mask file from fMRI data in 3D or 4D ndarrays.

    Compute and write the mask of an image based on the grey level
    This is based on an heuristic proposed by T.Nichols:
    find the least dense point of the histogram, between fractions
    m and M of the total image histogram.

    In case of failure, it is usually advisable to increase m.

    Parameters
    ----------
    mean_volume : 3D ndarray
        mean EPI image, used to compute the threshold for the mask.
    reference_volume: 3D ndarray, optional
        reference volume used to compute the mask. If none is give, the
        mean volume is used.
    m : float, optional
        lower fraction of the histogram to be discarded.
    M: float, optional
        upper fraction of the histogram to be discarded.
    cc: boolean, optional
        if cc is True, only the largest connect component is kept.
    opening: int, optional
        if opening is larger than 0, an morphological opening is performed,
        to keep only large structures. This step is useful to remove parts of
        the skull that might have been included.
    exclude_zeros: boolean, optional
        Consider zeros as missing values for the computation of the
        threshold. This option is useful if the images have been
        resliced with a large padding of zeros.

    Returns
    -------
    mask : 3D boolean ndarray
        The brain mask
    """
    if reference_volume is None:
        reference_volume = mean_volume
    sorted_input = np.sort(mean_volume.reshape(-1))
    if exclude_zeros:
        sorted_input = sorted_input[sorted_input != 0]
    limiteinf = np.floor(m * len(sorted_input))
    limitesup = np.floor(M * len(sorted_input))

    delta = sorted_input[limiteinf + 1:limitesup + 1] \
            - sorted_input[limiteinf:limitesup]
    ia = delta.argmax()
    threshold = 0.5 * (sorted_input[ia + limiteinf]
                        + sorted_input[ia + limiteinf + 1])

    mask = (reference_volume >= threshold)

    if cc:
        mask = largest_cc(mask)

    if opening > 0:
        mask = ndimage.binary_opening(mask.astype(np.int),
                                        iterations=opening)
    return mask.astype(bool)


def compute_mask_sessions(session_images, m=0.2, M=0.9, cc=1, threshold=0.5,
                          exclude_zeros=False, return_mean=False, opening=2):
    """ Compute a common mask for several sessions of fMRI data.

        Uses the mask-finding algorithmes to extract masks for each
        session, and then keep only the main connected component of the
        a given fraction of the intersection of all the masks.


    Parameters
    ----------
    session_images : list of (list of strings) or nipy image objects
        A list of images/list of nifti filenames. Each inner list/image
        represents a session.
    m : float, optional
        lower fraction of the histogram to be discarded.
    M: float, optional
        upper fraction of the histogram to be discarded.
    cc: boolean, optional
        if cc is True, only the largest connect component is kept.
    threshold : float, optional
        the inter-session threshold: the fraction of the
        total number of session in for which a voxel must be in the
        mask to be kept in the common mask.
        threshold=1 corresponds to keeping the intersection of all
        masks, whereas threshold=0 is the union of all masks.
    exclude_zeros: boolean, optional
        Consider zeros as missing values for the computation of the
        threshold. This option is useful if the images have been
        resliced with a large padding of zeros.
    return_mean: boolean, optional
        if return_mean is True, the mean image accross subjects is
        returned.
    opening: int, optional,
             size of  the morphological opening

    Returns
    -------
    mask : 3D boolean ndarray
        The brain mask
    mean : 3D float array
        The mean image
    """
    mask, mean = None, None
    for index, session in enumerate(session_images):
        if hasattr(session, 'get_data'):
            mean = session.get_data()
            if mean.ndim > 3:
                mean = mean.mean(-1)
            this_mask = compute_mask(mean, None, m=m, M=M, cc=cc,
                        opening=opening, exclude_zeros=exclude_zeros)
            if return_mean:
                this_mask = this_mask, mean
        else:
            this_mask = compute_mask_files(
                session, m=m, M=M, cc=cc, exclude_zeros=exclude_zeros,
                return_mean=return_mean, opening=opening)
        if return_mean:
            this_mask, this_mean = this_mask
            if mean is None:
                mean = this_mean.astype(np.float)
            else:
                mean += this_mean
        this_mask = this_mask.astype(np.int8)
        if mask is None:
            mask = this_mask
        else:
            mask += this_mask
        # Free memory early
        del this_mask

    # Take the "half-intersection", i.e. all the voxels that fall within
    # 50% of the individual masks.
    mask = (mask > threshold * len(list(session_images)))

    if cc:
        # Select the largest connected component (each mask is
        # connect, but the half-interesection may not be):
        mask = largest_cc(mask)
    mask = mask.astype(np.bool)

    if return_mean:
        # Divide by the number of sessions
        mean /= len(session_images)
        return mask, mean

    return mask


def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
    """
    Given a list of input mask images, generate the output image which
    is the the threshold-level intersection of the inputs

    Parameters
    ----------
    input_masks: list of strings or ndarrays
        paths of the input images nsubj set as len(input_mask_files), or
        individual masks.
    output_filename, string:
        Path of the output image, if None no file is saved.
    threshold: float within [0, 1[, optional
        gives the level of the intersection.
        threshold=1 corresponds to keeping the intersection of all
        masks, whereas threshold=0 is the union of all masks.
    cc: bool, optional
        If true, extract the main connected component

    Returns
    -------
    grp_mask, boolean array of shape the image shape
    """
    grp_mask = None
    if threshold > 1:
        raise ValueError('The threshold should be < 1')
    if threshold < 0:
        raise ValueError('The threshold should be > 0')
    threshold = min(threshold, 1 - 1.e-7)

    for this_mask in input_masks:
        if isinstance(this_mask, basestring):
            # We have a filename
            this_mask = load(this_mask).get_data()
        if grp_mask is None:
            grp_mask = this_mask.copy().astype(np.int)
        else:
            # If this_mask is floating point and grp_mask is integer, numpy 2
            # casting rules raise an error for in-place addition.
            # Hence we do it long-hand.
            # XXX should the masks be coerced to int before addition?
            grp_mask = grp_mask + this_mask

    grp_mask = grp_mask > (threshold * len(list(input_masks)))

    if np.any(grp_mask > 0) and cc:
        grp_mask = largest_cc(grp_mask)

    if output_filename is not None:
        if isinstance(input_masks[0], basestring):
            nim = load(input_masks[0])
            header = nim.get_header()
            affine = nim.get_affine()
        else:
            header = dict()
            affine = np.eye(4)
        header['descrip'] = 'mask image'
        output_image = nifti1.Nifti1Image(grp_mask.astype(np.uint8),
                                            affine=affine,
                                            header=header,
                                         )
        save(output_image, output_filename)

    return grp_mask > 0


###############################################################################
# Time series extraction
###############################################################################

def series_from_mask(filenames, mask, dtype=np.float32,
                     smooth=False, ensure_finite=True):
    """ Read the time series from the given sessions filenames, using the mask.

    Parameters
    -----------
    filenames: list of 3D nifti file names, or 4D nifti filename.
               Files are grouped by session.
    mask: 3d ndarray
          3D mask array: true where a voxel should be used.
    smooth: False or float, optional
            If smooth is not False, it gives the size, in voxel of the
            spatial smoothing to apply to the signal.
    ensure_finite: boolean, optional
            If ensure_finite is True, the non-finite values (NaNs and infs)
            found in the images will be replaced by zeros

    Returns
    --------
    session_series: ndarray
            3D array of time course: (session, voxel, time)
    header: header object
            The header of the first file.

    Notes
    -----
    When using smoothing, ensure_finite should be True: as elsewhere non
    finite values will spread accross the image.
    """
    assert len(filenames) != 0, (
        'filenames should be a file name or a list of file names, '
        '%s (type %s) was passed' % (filenames, type(filenames)))
    mask = mask.astype(np.bool)
    if smooth:
        # Convert from a sigma to a FWHM:
        smooth /= np.sqrt(8 * np.log(2))
    if isinstance(filenames, basestring):
        # We have a 4D nifti file
        data_file = load(filenames)
        header = data_file.get_header()
        series = data_file.get_data()
        if ensure_finite:
            # SPM tends to put NaNs in the data outside the brain
            series[np.logical_not(np.isfinite(series))] = 0
        series = series.astype(dtype)
        affine = data_file.get_affine()[:3, :3]
        del data_file
        if isinstance(series, np.memmap):
            series = np.asarray(series).copy()
        if smooth:
            vox_size = np.sqrt(np.sum(affine ** 2, axis=0))
            smooth_sigma = smooth / vox_size
            for this_volume in np.rollaxis(series, -1):
                this_volume[...] = ndimage.gaussian_filter(this_volume,
                                                        smooth_sigma)
        series = series[mask]
    else:
        nb_time_points = len(list(filenames))
        series = np.zeros((mask.sum(), nb_time_points), dtype=dtype)
        for index, filename in enumerate(filenames):
            data_file = load(filename)
            data = data_file.get_data()
            if ensure_finite:
                # SPM tends to put NaNs in the data outside the brain
                data[np.logical_not(np.isfinite(data))] = 0
            data = data.astype(dtype)
            if smooth is not False:
                affine = data_file.get_affine()[:3, :3]
                vox_size = np.sqrt(np.sum(affine ** 2, axis=0))
                smooth_sigma = smooth / vox_size
                data = ndimage.gaussian_filter(data, smooth_sigma)

            series[:, index] = data[mask]
            # Free memory early
            del data
            if index == 0:
                header = data_file.get_header()

    return series, header



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from .graph import (Graph, WeightedGraph, wgraph_from_coo_matrix,
                    wgraph_from_adjacency, complete_graph, mst, knn, eps_nn,
                    lil_cc, graph_3d_grid, wgraph_from_3d_grid,
                    concatenate_graphs)

from nipy.testing import Tester
test = Tester().test
bench = Tester().bench



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Support utilities for FIAC example, mostly path management.

The purpose of separating these is to keep the main example code as readable as
possible and focused on the experimental modeling and analysis, rather than on
local file management issues.

Requires matplotlib
"""

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------
from __future__ import print_function # Python 2/3 compatibility

# Stdlib
import os
from os import makedirs, listdir
from os.path import exists, abspath, isdir, join as pjoin, splitext
import csv
try:
    from StringIO import StringIO # Python 2
except ImportError:
    from io import StringIO # Python 3

# Third party
import numpy as np
from matplotlib.mlab import csv2rec, rec2csv

# From NIPY
from nipy.io.api import load_image

#-----------------------------------------------------------------------------
# Globals
#-----------------------------------------------------------------------------

# We assume that there is a directory holding the data and it's local to this
# code.  Users can either keep a copy here or a symlink to the real location on
# disk of the data.
DATADIR = 'fiac_data'

# Sanity check
if not os.path.isdir(DATADIR):
    e="The data directory %s must exist and contain the FIAC data." % DATADIR
    raise IOError(e)

#-----------------------------------------------------------------------------
# Classes and functions
#-----------------------------------------------------------------------------

# Path management utilities
def load_image_fiac(*path):
    """Return a NIPY image from a set of path components.
    """
    return load_image(pjoin(DATADIR, *path))


def subj_des_con_dirs(design, contrast, nsub=16):
    """Return a list of subject directories with this `design` and `contrast`

    Parameters
    ----------
    design : {'event', 'block'}
    contrast : str
    nsub : int, optional
        total number of subjects

    Returns
    -------
    con_dirs : list
        list of directories matching `design` and `contrast`
    """
    rootdir = DATADIR
    con_dirs = []
    for s in range(nsub):
        f = pjoin(rootdir, "fiac_%02d" % s, design, "fixed", contrast)
        if isdir(f):
            con_dirs.append(f)
    return con_dirs


def path_info_run(subj, run):
    """Construct path information dict for current subject/run.

    Parameters
    ----------
    subj : int
        subject number (0..15 inclusive)
    run : int
        run number (1..4 inclusive).

    Returns
    -------
    path_dict : dict
        a dict with all the necessary path-related keys, including 'rootdir',
        and 'design', where 'design' can have values 'event' or 'block'
        depending on which type of run this was for subject no `subj` and run no
        `run`
    """
    path_dict = {'subj': subj, 'run': run}
    if exists(pjoin(DATADIR, "fiac_%(subj)02d",
                    "block", "initial_%(run)02d.csv") % path_dict):
        path_dict['design'] = 'block'
    else:
        path_dict['design'] = 'event'
    rootdir = pjoin(DATADIR, "fiac_%(subj)02d", "%(design)s") % path_dict
    path_dict['rootdir'] = rootdir
    return path_dict


def path_info_design(subj, design):
    """Construct path information dict for subject and design.

    Parameters
    ----------
    subj : int
        subject number (0..15 inclusive)
    design : {'event', 'block'}
        type of design

    Returns
    -------
    path_dict : dict
        having keys 'rootdir', 'subj', 'design'
    """
    path_dict = {'subj': subj, 'design': design}
    rootdir = pjoin(DATADIR, "fiac_%(subj)02d", "%(design)s") % path_dict
    path_dict['rootdir'] = rootdir
    return path_dict


def results_table(path_dict):
    """ Return precalculated results images for subject info in `path_dict`

    Parameters
    ----------
    path_dict : dict
        containing key 'rootdir'

    Returns
    -------
    rtab : dict
        dict with keys given by run directories for this subject, values being a
        list with filenames of effect and sd images.
    """
    # Which runs correspond to this design type?
    rootdir = path_dict['rootdir']
    runs = filter(lambda f: isdir(pjoin(rootdir, f)),
                  ['results_%02d' % i for i in range(1,5)] )

    # Find out which contrasts have t-statistics,
    # storing the filenames for reading below

    results = {}

    for rundir in runs:
        rundir = pjoin(rootdir, rundir)
        for condir in listdir(rundir):
            for stat in ['sd', 'effect']:
                fname_effect = abspath(pjoin(rundir, condir, 'effect.nii'))
                fname_sd = abspath(pjoin(rundir, condir, 'sd.nii'))
            if exists(fname_effect) and exists(fname_sd):
                results.setdefault(condir, []).append([fname_effect,
                                                       fname_sd])
    return results


def get_experiment_initial(path_dict):
    """Get the record arrays for the experimental/initial designs.

    Parameters
    ----------
    path_dict : dict
        containing key 'rootdir', 'run', 'subj'

    Returns
    -------
    experiment, initial : Two record arrays.

    """
    # The following two lines read in the .csv files
    # and return recarrays, with fields
    # experiment: ['time', 'sentence', 'speaker']
    # initial: ['time', 'initial']

    rootdir = path_dict['rootdir']
    if not exists(pjoin(rootdir, "experiment_%(run)02d.csv") % path_dict):
        e = "can't find design for subject=%(subj)d,run=%(subj)d" % path_dict
        raise IOError(e)

    experiment = csv2rec(pjoin(rootdir, "experiment_%(run)02d.csv") % path_dict)
    initial = csv2rec(pjoin(rootdir, "initial_%(run)02d.csv") % path_dict)

    return experiment, initial


def get_fmri(path_dict):
    """Get the images for a given subject/run.

    Parameters
    ----------
    path_dict : dict
        containing key 'rootdir', 'run'

    Returns
    -------
    fmri : ndarray
    anat : NIPY image
    """
    fmri_im = load_image(
        pjoin("%(rootdir)s/swafunctional_%(run)02d.nii") % path_dict)
    return fmri_im


def ensure_dir(*path):
    """Ensure a directory exists, making it if necessary.

    Returns the full path."""
    dirpath = pjoin(*path)
    if not isdir(dirpath):
        makedirs(dirpath)
    return dirpath


def output_dir(path_dict, tcons, fcons):
    """Get (and make if necessary) directory to write output into.

    Parameters
    ----------
    path_dict : dict
        containing key 'rootdir', 'run'
    tcons : sequence of str
        t contrasts
    fcons : sequence of str
        F contrasts
    """
    rootdir = path_dict['rootdir']
    odir = pjoin(rootdir, "results_%(run)02d" % path_dict)
    ensure_dir(odir)
    for n in tcons:
        ensure_dir(odir,n)
    for n in fcons:
        ensure_dir(odir,n)
    return odir


def test_sanity():
    from nipy.modalities.fmri import design, hrf
    import nipy.modalities.fmri.fmristat.hrf as fshrf
    from nipy.modalities.fmri.fmristat.tests import FIACdesigns
    from nipy.modalities.fmri.fmristat.tests.test_FIAC import matchcol
    from nipy.algorithms.statistics import formula
    from nose.tools import assert_true

    """
    Single subject fitting of FIAC model
    """

    # Based on file
    # subj3_evt_fonc1.txt
    # subj3_bloc_fonc3.txt

    for subj, run, design_type in [(3, 1, 'event'), (3, 3, 'block')]:
        nvol = 191
        TR = 2.5
        Tstart = 1.25

        volume_times = np.arange(nvol)*TR + Tstart
        volume_times_rec = formula.make_recarray(volume_times, 't')

        path_dict = {'subj':subj, 'run':run}
        if exists(pjoin(DATADIR, "fiac_%(subj)02d",
                        "block", "initial_%(run)02d.csv") % path_dict):
            path_dict['design'] = 'block'
        else:
            path_dict['design'] = 'event'

        experiment = csv2rec(pjoin(DATADIR, "fiac_%(subj)02d", "%(design)s", "experiment_%(run)02d.csv")
                             % path_dict)
        initial = csv2rec(pjoin(DATADIR, "fiac_%(subj)02d", "%(design)s", "initial_%(run)02d.csv")
                                % path_dict)

        X_exper, cons_exper = design.event_design(experiment,
                                                  volume_times_rec,
                                                  hrfs=fshrf.spectral)
        X_initial, _ = design.event_design(initial,
                                           volume_times_rec,
                                           hrfs=[hrf.glover])
        X, cons = design.stack_designs((X_exper, cons_exper), (X_initial, {}))

        # Get original fmristat design
        Xf = FIACdesigns.fmristat[design_type]
        # Check our new design can be closely matched to the original
        for i in range(X.shape[1]):
            # Columns can be very well correlated negatively or positively
            assert_true(abs(matchcol(X[:,i], Xf)[1]) > 0.999)


def rewrite_spec(subj, run, root = "/home/jtaylo/FIAC-HBM2009"):
    """
    Take a FIAC specification file and get two specifications
    (experiment, begin).

    This creates two new .csv files, one for the experimental
    conditions, the other for the "initial" confounding trials that
    are to be modelled out. 

    For the block design, the "initial" trials are the first
    trials of each block. For the event designs, the 
    "initial" trials are made up of just the first trial.

    """

    if exists(pjoin("%(root)s", "fiac%(subj)d", "subj%(subj)d_evt_fonc%(run)d.txt") % {'root':root, 'subj':subj, 'run':run}):
        designtype = 'evt'
    else:
        designtype = 'bloc'

    # Fix the format of the specification so it is
    # more in the form of a 2-way ANOVA

    eventdict = {1:'SSt_SSp', 2:'SSt_DSp', 3:'DSt_SSp', 4:'DSt_DSp'}
    s = StringIO()
    w = csv.writer(s)
    w.writerow(['time', 'sentence', 'speaker'])

    specfile = pjoin("%(root)s", "fiac%(subj)d", "subj%(subj)d_%(design)s_fonc%(run)d.txt") % {'root':root, 'subj':subj, 'run':run, 'design':designtype}
    d = np.loadtxt(specfile)
    for row in d:
        w.writerow([row[0]] + eventdict[row[1]].split('_'))
    s.seek(0)
    d = csv2rec(s)

    # Now, take care of the 'begin' event
    # This is due to the FIAC design

    if designtype == 'evt':
        b = np.array([(d[0]['time'], 1)], np.dtype([('time', np.float),
                                                    ('initial', np.int)]))
        d = d[1:]
    else:
        k = np.equal(np.arange(d.shape[0]) % 6, 0)
        b = np.array([(tt, 1) for tt in d[k]['time']], np.dtype([('time', np.float),
                                                                 ('initial', np.int)]))
        d = d[~k]

    designtype = {'bloc':'block', 'evt':'event'}[designtype]

    fname = pjoin(DATADIR, "fiac_%(subj)02d", "%(design)s", "experiment_%(run)02d.csv") % {'root':root, 'subj':subj, 'run':run, 'design':designtype}
    rec2csv(d, fname)
    experiment = csv2rec(fname)

    fname = pjoin(DATADIR, "fiac_%(subj)02d", "%(design)s", "initial_%(run)02d.csv") % {'root':root, 'subj':subj, 'run':run, 'design':designtype}
    rec2csv(b, fname)
    initial = csv2rec(fname)

    return d, b


def compare_results(subj, run, other_root, mask_fname):
    """ Find and compare calculated results images from a previous run

    This scipt checks that another directory containing results of this same
    analysis are similar in the sense of numpy ``allclose`` within a brain mask.

    Parameters
    ----------
    subj : int
        subject number (0..4, 6..15)
    run : int
        run number (1..4)
    other_root : str
        path to previous run estimation
    mask_fname:
        path to a mask image defining area in which to compare differences
    """
    # Get information for this subject and run
    path_dict = path_info_run(subj, run)
    # Get mask
    msk = load_image(mask_fname).get_data().copy().astype(bool)
    # Get results directories for this run
    rootdir = path_dict['rootdir']
    res_dir = pjoin(rootdir, 'results_%02d' % run)
    if not isdir(res_dir):
        return
    for dirpath, dirnames, filenames in os.walk(res_dir):
        for fname in filenames:
            froot, ext = splitext(fname)
            if froot in ('effect', 'sd', 'F', 't'):
                this_fname = pjoin(dirpath, fname)
                other_fname = this_fname.replace(DATADIR, other_root)
                if not exists(other_fname):
                    print(this_fname, 'present but ', other_fname, 'missing')
                    continue
                this_arr = load_image(this_fname).get_data()
                other_arr = load_image(other_fname).get_data()
                ok = np.allclose(this_arr[msk], other_arr[msk])
                if not ok and froot in ('effect', 'sd', 't'): # Maybe a sign flip
                    ok = np.allclose(this_arr[msk], -other_arr[msk])
                if not ok:
                    print('Difference between', this_fname, other_fname)


def compare_all(other_root, mask_fname):
    """ Run results comparison for all subjects and runs """
    for subj in range(5) + range(6, 16):
        for run in range(1, 5):
            compare_results(subj, run, other_root, mask_fname)



""" Utilities to skip tests
"""

import sys
import inspect

def skip_if_running_nose(msg=None):
    """ Raise a SkipTest if we appear to be running the nose test loader.

    Parameters
    ==========
    msg: string, optional
        The message issued when SkipTest is raised
    """
    if not 'nose' in sys.modules:
        return
    try:
        import nose
    except ImportError:
        return
    # Now check that we have the loader in the call stask
    stack = inspect.stack()
    from nose import loader
    loader_file_name = loader.__file__
    if loader_file_name.endswith('.pyc'):
        loader_file_name = loader_file_name[:-1]
    for frame, file_name, line_num, func_name, line, number in stack:
        if file_name == loader_file_name:
            if msg is not None:
                raise nose.SkipTest(msg)
            else:
                raise nose.SkipTest






# emacs: -*- coding: utf-8; mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
#
# sampledoc documentation build configuration file, created by
# sphinx-quickstart on Tue Jun  3 12:40:24 2008.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# The contents of this file are pickled, so don't put values in the namespace
# that aren't pickleable (module imports are okay, they're removed automatically).
#
# All configuration values have a default value; values that are commented out
# serve to show the default value.

import sys, os

# If your extensions are in another directory, add it here. If the directory
# is relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
sys.path.append(os.path.abspath('sphinxext'))

# Get project related strings.  Please do not change this line to use
# execfile because execfile is not available in Python 3
_info_fname = os.path.join('..', 'nipy', 'info.py')
rel = {}
exec(open(_info_fname, 'rt').read(), {}, rel)

# Import support for ipython console session syntax highlighting (lives
# in the sphinxext directory defined above)
import ipython_console_highlighting

# General configuration
# ---------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.doctest',
              'sphinx.ext.pngmath',
              'sphinx.ext.autosummary',
              'inheritance_diagram',
              'numpy_ext.numpydoc',
              'matplotlib.sphinxext.plot_directive',
              'matplotlib.sphinxext.only_directives', # needed for above
              ]

# Autosummary on
autosummary_generate=True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General substitutions.
project = 'nipy'

#copyright = ':ref:`2005-2010, Neuroimaging in Python team. <nipy-software-license>`'
copyright = '2005-2012, Neuroimaging in Python team'

# The default replacements for |version| and |release|, also used in various
# other places throughout the built documents.
#
# The short X.Y version.
version = rel['__version__']
# The full version, including alpha/beta/rc tags.
release = version

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
unused_docs = []

# List of directories, relative to source directories, that shouldn't
# be searched for source files.
# exclude_trees = []

# what to put into API doc (just class doc, just init, or both)
autoclass_content = 'class'

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'


# Options for HTML output
# -----------------------
#
# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'sphinxdoc'

# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
html_style = 'nipy.css'

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = 'NIPY Documentation'

# The name of an image file (within the static path) to place at the top of
# the sidebar.
#html_logo = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Content template for the index page.
html_index = 'index.html'

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {'index': 'indexsidebar.html'}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_use_modindex = True

# If true, the reST sources are included in the HTML build as _sources/<name>.
html_copy_source = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = project

# Options for LaTeX output
# ------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, document class
# [howto/manual]).

latex_documents = [
  ('documentation', 'nipy.tex', 'Neuroimaging in Python Documentation',
   ur'Neuroimaging in Python team.','manual'),
  ]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
latex_use_parts = True

# Additional stuff for the LaTeX preamble.
latex_preamble = """
   \usepackage{amsmath}
   \usepackage{amssymb}
   % Uncomment these two if needed
   %\usepackage{amsfonts}
   %\usepackage{txfonts}
"""

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
latex_use_modindex = True



#!/usr/bin/env python

import numpy as np
from scipy import special

from ..routines import median, mahalanobis, gamln, psi

from nose.tools import assert_true
from numpy.testing import assert_almost_equal, assert_equal, TestCase


class TestAll(TestCase):

    def test_median(self):
        x = np.random.rand(100)
        assert_almost_equal(median(x), np.median(x))

    def test_median2(self):
        x = np.random.rand(101)
        assert_equal(median(x), np.median(x))

    def test_median3(self):
        x = np.random.rand(10, 30, 11)
        assert_almost_equal(np.squeeze(median(x,axis=1)), np.median(x,axis=1))

    def test_mahalanobis(self):
        x = np.random.rand(100) / 100
        A = np.random.rand(100, 100) / 100
        A = np.dot(A.transpose(), A) + np.eye(100)
        mah = np.dot(x, np.dot(np.linalg.inv(A), x))
        assert_almost_equal(mah, mahalanobis(x, A), decimal=1) 

    def test_mahalanobis2(self):
        x = np.random.rand(100,3,4)
        Aa = np.zeros([100,100,3,4])
        for i in range(3):
            for j in range(4):
                A = np.random.rand(100,100)
                A = np.dot(A.T, A)
                Aa[:,:,i,j] = A
        i = np.random.randint(3)
        j = np.random.randint(4)
        mah = np.dot(x[:,i,j], np.dot(np.linalg.inv(Aa[:,:,i,j]), x[:,i,j]))
        f_mah = (mahalanobis(x, Aa))[i,j]
        assert_true(np.allclose(mah, f_mah))

    def test_gamln(self):
        for x in (0.01+100*np.random.random(50)):
            scipy_gamln = special.gammaln(x)
            my_gamln = gamln(x)
            assert_almost_equal(scipy_gamln, my_gamln)

    def test_psi(self):
        for x in (0.01+100*np.random.random(50)):
            scipy_psi = special.psi(x)
            my_psi = psi(x)
            assert_almost_equal(scipy_psi, my_psi)


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the discrete_domain utilities.

Caveat assumes that the MNI template image is available at
in ~/.nipy/tests/data
"""

import numpy as np
from ..mroi import subdomain_from_array, subdomain_from_balls
from ..discrete_domain import domain_from_binary_array

from numpy.testing import assert_equal

shape = (5, 6, 7)


###########################################################
# SubDomains tests
###########################################################

def make_subdomain():
    """Create a multiple ROI instance
    """
    labels = np.zeros(shape)
    labels[4:, 5:, 6:] = 1
    labels[:2, 0:2, 0:2] = 2
    labels[:2, 5:, 6:] = 3
    labels[:2, 0:2, 6:] = 4
    labels[4:, 0:2, 6:] = 5
    labels[4:, 0:2, 0:2] = 6
    labels[4:, 5:, 0:2] = 7
    labels[:2, 5:, 0:2] = 8
    mroi = subdomain_from_array(labels - 1, affine=None)
    return mroi


def test_subdomain():
    """Test basic construction of multiple_roi
    """
    mroi = make_subdomain()
    assert_equal(mroi.k, 8)


def test_subdomain2():
    """Test mroi.size
    """
    mroi = make_subdomain()
    assert_equal(len(mroi.get_size()), 8)
    for k in mroi.get_id():
        assert_equal(mroi.get_size(k),
                     np.sum(mroi.label == mroi.select_id(k)))


def test_copy_subdomain():
    """Test basic construction of multiple_roi
    """
    mroi = make_subdomain()
    foo_feature = [[i] * j for i, j in enumerate(mroi.get_size())]
    foo_roi_feature = np.arange(mroi.k)
    mroi.set_feature('a', foo_feature)
    mroi.set_roi_feature('b', foo_roi_feature)
    mroi_copy = mroi.copy()
    # check some properties of mroi
    assert_equal(mroi.k, 8)
    for k in mroi.get_id():
        assert_equal(mroi.get_feature('a', k), foo_feature[mroi.select_id(k)])
    assert_equal(mroi.get_roi_feature('b'), foo_roi_feature)
    # delete mroi
    del mroi
    # check mroi_copy
    assert_equal(mroi_copy.k, 8)
    for k in mroi_copy.get_id():
        assert_equal(mroi_copy.get_feature('a', k),
                     foo_feature[mroi_copy.select_id(k)])
    assert_equal(mroi_copy.get_roi_feature('b'), foo_roi_feature)


def test_select_roi():
    """
    """
    mroi = make_subdomain()
    aux = np.random.randn(np.prod(shape))
    data = [aux[mroi.label == k] for k in range(8)]
    mroi.set_feature('data', data)
    mroi.set_roi_feature('data_mean', range(8))
    mroi.select_roi([0])
    assert(mroi.k == 1)
    assert_equal(mroi.get_roi_feature('data_mean', 0), 0)

def test_roi_features():
    """
    """
    mroi = make_subdomain()
    dshape = (8, 3)
    data = np.random.randn(*dshape)
    mroi.set_roi_feature('data_mean', data)
    assert mroi.roi_features['data_mean'].shape == dshape

def test_subdomain_feature():
    """Test the basic construction of features
    """
    mroi = make_subdomain()
    aux = np.random.randn(np.prod(shape))
    data = [aux[mroi.label == k] for k in range(8)]
    mroi.set_feature('data', data)
    assert_equal(mroi.features['data'][0], data[0])


def test_sd_integrate():
    """Test the integration
    """
    mroi = make_subdomain()
    aux = np.random.randn(np.prod(shape))
    data = [aux[mroi.label == k] for k in range(8)]
    mroi.set_feature('data', data)
    sums = mroi.integrate('data')
    for k in range(8):
        assert_equal(sums[k], np.sum(data[k]))


def test_sd_integrate2():
    """Test the integration
    """
    mroi = make_subdomain()
    for k in mroi.get_id():
        assert_equal(mroi.get_volume(k), mroi.integrate(id=k))
    volume_from_integration = mroi.integrate()
    volume_from_feature = mroi.get_volume()
    for i in range(mroi.k):
        assert_equal(volume_from_feature[i], volume_from_integration[i])


def test_sd_representative():
    """Test the computation of representative features
    """
    mroi = make_subdomain()
    data = [[k] * mroi.get_size(k) for k in mroi.get_id()]
    mroi.set_feature('data', data)
    sums = mroi.representative_feature('data')
    for k in mroi.get_id():
        assert_equal(sums[mroi.select_id(k)], k)


def test_sd_from_ball():
    dom = domain_from_binary_array(np.ones((10, 10)))
    radii = np.array([2, 2, 2])
    positions = np.array([[3, 3], [3, 7], [7, 7]])
    subdomain = subdomain_from_balls(dom, positions, radii)
    assert_equal(subdomain.k, 3)
    assert_equal(subdomain.get_size(), np.array([9, 9, 9]))


def test_set_feature():
    """Test the feature building capability
    """
    mroi = make_subdomain()
    data = np.random.randn(np.prod(shape))
    feature_data = [data[mroi.select_id(k, roi=False)]
                    for k in mroi.get_id()]
    mroi.set_feature('data', feature_data)
    get_feature_output = mroi.get_feature('data')
    assert_equal([len(k) for k in mroi.get_feature('data')],
                 mroi.get_size())
    for k in mroi.get_id():
        assert_equal(mroi.get_feature('data', k),
                     data[mroi.select_id(k, roi=False)])
        assert_equal(get_feature_output[k],
                     data[mroi.select_id(k, roi=False)])


def test_set_feature2():
    """
    """
    mroi = make_subdomain()
    data = np.random.randn(np.prod(shape))
    feature_data = [data[mroi.select_id(k, roi=False)]
                    for k in mroi.get_id()]
    mroi.set_feature('data', feature_data)
    mroi.set_feature('data', np.asarray([1000]), id=0, override=True)
    assert_equal(mroi.get_feature('data', 0), [1000])


def test_get_coord():
    """
    """
    mroi = make_subdomain()
    for k in mroi.get_id():
        assert_equal(mroi.get_coord(k),
                     mroi.domain.coord[mroi.select_id(k, roi=False)])

if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Package contains generic functions for data input/output. This includes
methods for accessing file systems and network resources.
"""
__docformat__ = 'restructuredtext'



# -*- Mode: Python -*-  Not really, but the syntax is close enough

"""
Very fast quantile computation using partial sorting.
Author: Alexis Roche.
"""

__version__ = '0.1'

import numpy as np
cimport numpy as np

cdef extern from "quantile.h":
    double quantile(double* data,
                    np.npy_intp size,
                    np.npy_intp stride,
                    double r,
                    int interp)

np.import_array()

# This is faster than scipy.stats.scoreatpercentile owing to partial
# sorting
def _quantile(X, double ratio, int interp=False, int axis=0):
    """
    Fast quantile computation using partial sorting. This function has
    similar behavior to `scipy.percentile` but runs significantly
    faster for large arrays.

    Parameters
    ----------
    X : array
      Input array. Will be internally converted into an array of
      doubles if needed.

    ratio : float
      A value in range [0, 1] defining the desired quantiles (the
      higher the ratio, the higher the quantiles).

    interp : boolean
      Determine whether quantiles are interpolated.

    axis : int
      Axis along which quantiles are computed.

    Output
    ------
    Y : array
      Array of quantiles
    """
    cdef double *x, *y
    cdef long int size, stride
    cdef np.flatiter itX, itY

    # Convert the input array to double if needed
    X = np.asarray(X, dtype='double')

    # Check the input ratio is in range (0,1)
    if ratio < 0 or ratio > 1:
        raise ValueError('ratio must be in range 0..1')

    # Allocate output array Y
    dims = list(X.shape)
    dims[axis] = 1
    Y = np.zeros(dims)

    # Set size and stride along specified axis
    size = X.shape[axis]
    stride = X.strides[axis] / sizeof(double)

    # Create array iterators
    itX = np.PyArray_IterAllButAxis(X, &axis)
    itY = np.PyArray_IterAllButAxis(Y, &axis)

    # Loop 
    while np.PyArray_ITER_NOTDONE(itX):
        x = <double*>np.PyArray_ITER_DATA(itX)
        y = <double*>np.PyArray_ITER_DATA(itY)
        y[0] = quantile(x, size, stride, ratio, interp) 
        np.PyArray_ITER_NEXT(itX)
        np.PyArray_ITER_NEXT(itY)

    return Y

# This is faster than numpy.stats
# due to the underlying algorithm that relies on
# partial sorting as opposed to full sorting.
def _median(X, axis=0):
    """
    Fast median computation using partial sorting. This function is
    similar to `numpy.median` but runs significantly faster for large
    arrays.

    Parameters
    ----------
    X : array
      Input array. Will be internally converted into an array of
      doubles if needed.

    axis : int
      Axis along which medians are computed.

    Output
    ------
    Y : array
      Array of medians
    """
    return _quantile(X, axis=axis, ratio=0.5, interp=True)




#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
This is an example where:

1. A sequence of fMRI volumes are loaded
2. An ROI mask is loaded
3. A design matrix describing all the effects related to the data is computed
4. A GLM is applied to all voxels in the ROI
5. A summary of the results is provided for certain contrasts
6. A plot of the HRF is provided for the mean response in the HRF
7. Fitted/adjusted response plots are provided

Needs matplotlib

Author : Bertrand Thirion, 2010
"""
print(__doc__)

from os import mkdir, getcwd, path

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nibabel import save, load

from nipy.modalities.fmri.design_matrix import dmtx_light
from nipy.modalities.fmri.experimental_paradigm import EventRelatedParadigm
from nipy.labs.utils.simul_multisubject_fmri_dataset import \
    surrogate_4d_dataset
from nipy.modalities.fmri.glm import GeneralLinearModel
import nipy.labs.spatial_models.mroi as mroi
from nipy.labs.spatial_models.discrete_domain import grid_domain_from_image

# Local import
from get_data_light import DATA_DIR, get_second_level_dataset

#######################################
# Simulation parameters
#######################################

# volume mask
mask_path = path.join(DATA_DIR, 'mask.nii.gz')
if not path.exists(mask_path):
    get_second_level_dataset()

mask = load(mask_path)
mask_array, affine = mask.get_data() > 0, mask.get_affine()

# timing
n_scans = 128
tr = 2.4

# paradigm
frametimes = np.linspace(0, (n_scans - 1) * tr, n_scans)
conditions = np.arange(20) % 2
onsets = np.linspace(5, (n_scans - 1) * tr - 10, 20)  # in seconds
hrf_model = 'canonical'
motion = np.cumsum(np.random.randn(n_scans, 6), 0)
add_reg_names = ['tx', 'ty', 'tz', 'rx', 'ry', 'rz']

# write directory
write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)

########################################
# Design matrix
########################################

paradigm = np.vstack(([conditions, onsets])).T
paradigm = EventRelatedParadigm(conditions, onsets)
X, names = dmtx_light(frametimes, paradigm, drift_model='cosine', hfcut=128,
                      hrf_model=hrf_model, add_regs=motion,
                      add_reg_names=add_reg_names)

########################################
# Create ROIs
########################################

positions = np.array([[60, -30, 5], [50, 27, 5]])
# in mm (here in the MNI space)
radii = np.array([8, 6])

domain = grid_domain_from_image(mask)
my_roi = mroi.subdomain_from_balls(domain, positions, radii)

# to save an image of the ROIs
save(my_roi.to_image(), path.join(write_dir, "roi.nii"))

#######################################
# Get the FMRI data
#######################################
fmri_data = surrogate_4d_dataset(mask=mask, dmtx=X)[0]
Y = fmri_data.get_data()[mask_array]

# artificially added signal in ROIs to make the example more meaningful
activation = 30 * (X.T[1] + .5 * X.T[0])
for (position, radius) in zip(positions, radii):
    Y[((domain.coord - position) ** 2).sum(1) < radius ** 2 + 1] += activation

########################################
# Perform a GLM analysis
########################################

# GLM fit
glm = GeneralLinearModel(X)
glm.fit(Y.T)

# specifiy the contrast [1 -1 0 ..]
contrast = np.hstack((1, -1, np.zeros(X.shape[1] - 2)))

# compute the constrast image related to it
zvals = glm.contrast(contrast).z_score()

########################################
# ROI-based analysis
########################################

# exact the time courses with ROIs
signal_feature = [Y[my_roi.select_id(id, roi=False)] for id in my_roi.get_id()]
my_roi.set_feature('signal', signal_feature)

# ROI average time courses
my_roi.set_roi_feature('signal_avg', my_roi.representative_feature('signal'))

# roi-level contrast average
contrast_feature = [zvals[my_roi.select_id(id, roi=False)]
                    for id in my_roi.get_id()]
my_roi.set_feature('contrast', contrast_feature)
my_roi.set_roi_feature('contrast_avg',
                       my_roi.representative_feature('contrast'))

########################################
# GLM analysis on the ROI average time courses
########################################

n_reg = len(names)
roi_tc = my_roi.get_roi_feature('signal_avg')
glm.fit(roi_tc.T)

plt.figure()
plt.subplot(1, 2, 1)
betas = glm.get_beta()
b1 = plt.bar(np.arange(n_reg - 1), betas[:-1, 0], width=.4, color='blue',
            label='region 1')
b2 = plt.bar(np.arange(n_reg - 1) + 0.3, betas[:- 1, 1], width=.4,
            color='red', label='region 2')
plt.xticks(np.arange(n_reg - 1), names[:-1], fontsize=10)
plt.legend()
plt.title('Parameter estimates \n for the roi time courses')

bx = plt.subplot(1, 2, 2)
my_roi.plot_feature('contrast', bx)

########################################
# fitted and adjusted response
########################################

res = np.hstack([x.resid for x in glm.results_.values()]).T
betas = np.hstack([x.theta for x in glm.results_.values()])
proj = np.eye(n_reg)
proj[2:] = 0
fit = np.dot(np.dot(betas.T, proj), X.T)

# plot it
plt.figure()
for k in range(my_roi.k):
    plt.subplot(my_roi.k, 1, k + 1)
    plt.plot(fit[k])
    plt.plot(fit[k] + res[k], 'r')
    plt.xlabel('time (scans)')
    plt.legend(('effects', 'adjusted'))

###########################################
# hrf for condition 1
############################################

fir_order = 6
X_fir, _ = dmtx_light(
    frametimes, paradigm, hrf_model='fir', drift_model='cosine',
    drift_order=3, fir_delays=np.arange(fir_order), add_regs=motion,
    add_reg_names=add_reg_names)
glm_fir = GeneralLinearModel(X_fir)
plt.figure()

for k in range(my_roi.k):
    # fit a glm on the ROI's time course
    glm_fir.fit(roi_tc[k])
    # access to the corresponding result structure
    res = list(glm_fir.results_.values())[0] # only one value in this case
    plt.subplot(1, my_roi.k, k + 1)

    # get the confidence intervals for the effects and plot them -condition 0
    conf_int = res.conf_int(cols=np.arange(fir_order)).squeeze()
    yerr = (conf_int[:, 1] - conf_int[:, 0]) / 2
    plt.errorbar(np.arange(fir_order), conf_int.mean(1), yerr=yerr)

    # get the confidence intervals for the effects and plot them -condition 1
    conf_int = res.conf_int(cols=np.arange(fir_order, 2 * fir_order)).squeeze()
    yerr = (conf_int[:, 1] - conf_int[:, 0]) / 2
    plt.errorbar(np.arange(fir_order), conf_int.mean(1), yerr=yerr)
    plt.legend(('condition c0', 'condition c1'))
    plt.title('estimated hrf shape')
    plt.xlabel('time(scans)')

plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from .transform import Transform
from .affine import apply_affine
from ._registration import _apply_polyaffine

TINY_SIGMA = 1e-200


class PolyAffine(Transform):

    def __init__(self, centers, affines, sigma, glob_affine=None):
        """
        centers: N times 3 array

        We are given a set of affine transforms T_i with centers x_i,
        all in homogeneous coordinates. The polyaffine transform is
        defined, up to a right composition with a global affine, as:

        T(x) = sum_i w_i(x) T_i x

        where w_i(x) = g(x-x_i)/Z(x) are normalized Gaussian weights
        that sum up to one for every x.
        """

        # Format input arguments
        self.centers = np.asarray(centers, dtype='double', order='C')
        self.sigma = np.zeros(3)
        self.sigma[:] = np.maximum(TINY_SIGMA, sigma)
        if hasattr(affines[0], 'as_affine'):
            affines = np.array([a.as_affine() for a in affines])
        else:
            affines = np.asarray(affines)
        if hasattr(glob_affine, 'as_affine'):
            self.glob_affine = glob_affine.as_affine()
        else:
            self.glob_affine = glob_affine

        # Cache a (N, 12) matrix containing the affines coefficients,
        # should be C-contiguous double.
        self._affines = np.zeros((len(self.centers), 12))
        self._affines[:] = np.reshape(affines[:, 0:3, :],
                                      (len(self.centers), 12))

    def affine(self, i):
        aff = np.eye(4)
        aff[0:3, :] = self._affines[i].reshape(3, 4)
        return aff

    def affines(self):
        return [self.affine(i) for i in range(len(self.centers))]

    def apply(self, xyz):
        """
        xyz is an (N, 3) array
        """
        # txyz should be double C-contiguous for the the cython
        # routine _apply_polyaffine
        if self.glob_affine == None:
            txyz = np.array(xyz, copy=True, dtype='double', order='C')
        else:
            txyz = apply_affine(self.glob_affine, xyz)
        _apply_polyaffine(txyz, self.centers, self._affines, self.sigma)
        return txyz

    def compose(self, other):
        """
        Compose this transform onto another

        Parameters
        ----------
        other : Transform
            transform that we compose onto

        Returns
        -------
        composed_transform : Transform
            a transform implementing the composition of self on `other`
        """
        # If other is not an Affine, use the generic compose method
        if not hasattr(other, 'as_affine'):
            return Transform(self.apply).compose(other)

        # Affine case: the result is a polyaffine transform with same
        # local affines
        if self.glob_affine == None:
            glob_affine = other.as_affine()
        else:
            glob_affine = np.dot(self.glob_affine, other.as_affine())

        return self.__class__(self.centers, self.affines(), self.sigma,
                              glob_affine=glob_affine)

    def left_compose(self, other):

        # If other is not an Affine, use the generic compose method
        if not hasattr(other, 'as_affine'):
            return Transform(other.apply).compose(self)

        # Affine case: the result is a polyaffine transform with same
        # global affine
        other_affine = other.as_affine()
        affines = [np.dot(other_affine, self.affine(i)) \
                       for i in range(len(self.centers))]
        return self.__class__(self.centers, affines, self.sigma,
                              glob_affine=self.glob_affine)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the design_matrix utilities.

Note that the tests just look whether the data produced has correct dimension,
not whether it is exact.
"""

import numpy as np

from ..experimental_paradigm import (EventRelatedParadigm, BlockParadigm,
                                     load_paradigm_from_csv_file)


def basic_paradigm():
    conditions = ['c0', 'c0', 'c0', 'c1', 'c1', 'c1', 'c2', 'c2', 'c2']
    onsets = [30, 70, 100, 10, 30, 90, 30, 40, 60]
    paradigm =  EventRelatedParadigm(conditions, onsets)
    return paradigm


def modulated_block_paradigm():
    conditions = ['c0', 'c0', 'c0', 'c1', 'c1', 'c1', 'c2', 'c2', 'c2']
    onsets = [30, 70, 100, 10, 30, 90, 30, 40, 60]
    duration = 5 + 5 * np.random.rand(len(onsets))
    values = np.random.rand(len(onsets))
    paradigm = BlockParadigm(conditions, onsets, duration, values)
    return paradigm


def modulated_event_paradigm():
    conditions = ['c0', 'c0', 'c0', 'c1', 'c1', 'c1', 'c2', 'c2', 'c2']
    onsets = [30, 70, 100, 10, 30, 90, 30, 40, 60]
    values = np.random.rand(len(onsets))
    paradigm = EventRelatedParadigm(conditions, onsets, values)
    return paradigm


def block_paradigm():
    conditions = ['c0', 'c0', 'c0', 'c1', 'c1', 'c1', 'c2', 'c2', 'c2']
    onsets = [30, 70, 100, 10, 30, 90, 30, 40, 60]
    duration = 5 * np.ones(9)
    paradigm = BlockParadigm (conditions, onsets, duration)
    return paradigm

def write_paradigm(paradigm, session):
    """Function to write a paradigm to a file and return the address
    """
    import tempfile
    csvfile = tempfile.mkdtemp() + '/paradigm.csv'
    paradigm.write_to_csv(csvfile, session)
    return csvfile

def test_read_paradigm():
    """ test that a paradigm is correctly read
    """
    session = 'sess'
    paradigm = block_paradigm()
    csvfile = write_paradigm(paradigm, session)
    read_paradigm = load_paradigm_from_csv_file(csvfile)[session]
    assert (read_paradigm.onset == paradigm.onset).all()

    paradigm = modulated_event_paradigm()
    csvfile = write_paradigm(paradigm, session)
    read_paradigm = load_paradigm_from_csv_file(csvfile)[session]
    assert (read_paradigm.onset == paradigm.onset).all()

    paradigm = modulated_block_paradigm()
    csvfile = write_paradigm(paradigm, session)
    read_paradigm = load_paradigm_from_csv_file(csvfile)[session]
    assert (read_paradigm.onset == paradigm.onset).all()

    paradigm = basic_paradigm()
    csvfile = write_paradigm(paradigm, session)
    read_paradigm = load_paradigm_from_csv_file(csvfile)[session]
    assert (read_paradigm.onset == paradigm.onset).all()


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# -*- Mode: Python -*-  Not really, but the syntax is close enough
"""
Two-level general linear model for group analyses.

Author: Alexis Roche, 2008.
"""

__version__ = '0.1'

# Includes
from fff cimport *

# Exports from fff_glm_twolevel.h 
cdef extern from "fff_glm_twolevel.h":

    ctypedef struct fff_glm_twolevel_EM:
        fff_vector* b
        double s2
        fff_vector* z
        fff_vector* vz

    fff_glm_twolevel_EM* fff_glm_twolevel_EM_new(size_t n, size_t p)
    void fff_glm_twolevel_EM_delete(fff_glm_twolevel_EM* thisone)
    void fff_glm_twolevel_EM_init(fff_glm_twolevel_EM* em)
    void fff_glm_twolevel_EM_run(fff_glm_twolevel_EM* em,
                                 fff_vector* y,
                                 fff_vector* vy, 
                                 fff_matrix* X,
                                 fff_matrix* PpiX,
                                 unsigned int niter)
    double fff_glm_twolevel_log_likelihood(fff_vector* y,
                                           fff_vector* vy,
                                           fff_matrix* X, 
                                           fff_vector* b,
                                           double s2,
                                           fff_vector* tmp)

# Initialize numpy
fffpy_import_array()
import_array()
import numpy as np

# Constants
DEF_NITER = 2

def em(ndarray Y, ndarray VY, ndarray X, ndarray C=None, int axis=0, int niter=DEF_NITER):
    """
    b, s2 = em(y, vy, X, C=None, axis=0, niter=DEF_NITER).

    Maximum likelihood regression in a mixed-effect GLM using the
    EM algorithm.

    C is the contrast matrix. Conventionally, C is p x q where p
    is the number of regressors. 
    
    OUTPUT: beta, s2
    beta -- array of parameter estimates
    s2 -- array of squared scale parameters.
    
    REFERENCE:
    Keller and Roche, ISBI 2008.
    """
    cdef size_t n, p
    cdef fff_vector *y, *vy, *b, *s2
    cdef fff_matrix *x, *ppx
    cdef fff_glm_twolevel_EM *em
    cdef fffpy_multi_iterator* multi

    # View on design matrix
    x = fff_matrix_fromPyArray(X)

    # Number of observations / regressors 
    n = x.size1
    p = x.size2

    # Compute the projected pseudo-inverse matrix
    if C == None:
        PpX =  np.linalg.pinv(X)
    else:
        A = np.linalg.inv(np.dot(X.transpose(), X)) # (p,p)
        B = np.linalg.inv(np.dot(np.dot(C.transpose(), A), C)) # (q,q)
        P = np.eye(p) - np.dot(np.dot(np.dot(A, C), B), C.transpose()) # (p,p)
        PpX = np.dot(np.dot(P, A), X.transpose()) # (p,n)
    ppx = fff_matrix_fromPyArray(PpX)

    # Allocate output arrays
    dims = [Y.shape[i] for i in range(Y.ndim)]
    dims[axis] = p
    B = np.zeros(dims)
    dims[axis] = 1
    S2 = np.zeros(dims)

    # Local structs
    em = fff_glm_twolevel_EM_new(n, p)

    # Create a new array iterator 
    multi = fffpy_multi_iterator_new(4, axis, <void*>Y, <void*>VY,
                     <void*>B, <void*>S2)

    # Create views
    y = multi.vector[0]
    vy = multi.vector[1]
    b = multi.vector[2]
    s2 = multi.vector[3]

        # Loop 
    while(multi.index < multi.size):
        fff_glm_twolevel_EM_init(em)
        fff_glm_twolevel_EM_run(em, y, vy, x, ppx, niter)
        fff_vector_memcpy(b, em.b)
        s2.data[0] = em.s2
        fffpy_multi_iterator_update(multi)
    
    # Free memory
    fff_matrix_delete(x)
    fff_matrix_delete(ppx)
    fffpy_multi_iterator_delete(multi)
    fff_glm_twolevel_EM_delete(em)

    # Return
    return B, S2




def log_likelihood(Y, VY, X, B, S2, int axis=0):
    """
    ll = log_likelihood(y, vy, X, b, s2, axis=0)
    Log likelihood in a mixed-effect GLM.
    OUTPUT: array
    REFERENCE:
    Keller and Roche, ISBI 2008.
    """
    cdef fff_vector *y, *vy, *b, *s2, *ll, *tmp
    cdef fff_matrix *x
    cdef fffpy_multi_iterator* multi
    
    # Allocate output array
    dims = [Y.shape[i] for i in range(Y.ndim)]
    dims[axis] = 1
    LL = np.zeros(dims)

    # View on design matrix
    x = fff_matrix_fromPyArray(X)

    # Local structure
    tmp = fff_vector_new(x.size1)

    # Multi iterator 
    multi = fffpy_multi_iterator_new(5, axis, <void*>Y, <void*>VY,
                     <void*>B, <void*>S2, <void*>LL)

    # View on iterable arrays
    y = multi.vector[0]
    vy = multi.vector[1]
    b = multi.vector[2]
    s2 = multi.vector[3]
    ll = multi.vector[4]

    # Loop 
    while(multi.index < multi.size):
        ll.data[0] = fff_glm_twolevel_log_likelihood(y, vy, x, b, s2.data[0], tmp)
        fffpy_multi_iterator_update(multi)

    # Free memory
    fff_matrix_delete(x)
    fff_vector_delete(tmp)
    fffpy_multi_iterator_delete(multi)

    # Return
    return LL


def log_likelihood_ratio(Y, VY, X, C, int axis=0, int niter=DEF_NITER):
    """
    lda = em(y, vy, X, C, axis=0, niter=DEF_NITER).
    """

    # Constrained log-likelihood
    B, S2 = em(Y, VY, X, C, axis, niter)
    ll0 = log_likelihood(Y, VY, X, B, S2, axis)

    # Unconstrained log-likelihood
    B, S2 = em(Y, VY, X, None, axis, niter)
    ll = log_likelihood(Y, VY, X, B, S2, axis)
    
    # -2 log R = 2*(ll-ll0)
    lda = 2*(ll-ll0)
    return np.maximum(lda, 0.0)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from . import onesample
from . import twosample
from . import glm_twolevel
from . import permutation_test
from . import spatial_relaxation_onesample

from nipy.testing import Tester
test = Tester().test
bench = Tester().bench



""" Testing arrays module
"""

import numpy as np

from ..arrays import strides_from

from numpy.testing import (assert_array_almost_equal,
                           assert_array_equal)

from nose.tools import assert_true, assert_equal, assert_raises


def test_strides_from():
    for shape in ((3,), (2,3), (2,3,4), (5,4,3,2)):
        for order in 'FC':
            for dtype in sum(np.sctypes.values(), []):
                if dtype is str:
                    dtype = 'S3'
                elif dtype is unicode:
                    dtype = 'U4'
                elif dtype is np.void:
                    continue
                exp = np.empty(shape, dtype=dtype, order=order).strides
                assert_equal(strides_from(shape, dtype, order), exp)
            assert_raises(ValueError, strides_from, shape, np.void, order)
            assert_raises(ValueError, strides_from, shape, str, order)
            assert_raises(ValueError, strides_from, shape, unicode, order)
    assert_raises(ValueError, strides_from, (3,2), 'f8', 'G')



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
This is an example where:

1. An sequence of fMRI volumes are simulated
2. A design matrix describing all the effects related to the data is computed
3. A GLM is applied to all voxels
4. A contrast image is created

Requires matplotlib

Author : Bertrand Thirion, 2010
"""
print(__doc__)

import os
import os.path as op

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nibabel import save, Nifti1Image

import nipy.modalities.fmri.design_matrix as dm
from nipy.labs.utils.simul_multisubject_fmri_dataset import \
     surrogate_4d_dataset
from nipy.modalities.fmri.glm import GeneralLinearModel
from nipy.modalities.fmri.experimental_paradigm import EventRelatedParadigm

#######################################
# Simulation parameters
#######################################

# volume mask
shape = (20, 20, 20)
affine = np.eye(4)

# Acquisition parameters: number of scans (n_scans) and volume repetition time
# value in seconds
n_scans = 128
tr = 2.4

# input paradigm information
frametimes = np.linspace(0, (n_scans - 1) * tr, n_scans)

# conditions are 0 1 0 1 0 1 ...
conditions = np.arange(20) % 2

# 20 onsets (in sec), first event 10 sec after the start of the first scan
onsets = np.linspace(5, (n_scans - 1) * tr - 10, 20)

# model with canonical HRF (could also be :
#   'canonical with derivative' or 'fir'
hrf_model = 'canonical'

# fake motion parameters to be included in the model
motion = np.cumsum(np.random.randn(n_scans, 6), 0)
add_reg_names = ['tx', 'ty', 'tz', 'rx', 'ry', 'rz']

########################################
# Design matrix
########################################

paradigm = EventRelatedParadigm(conditions, onsets)
X, names = dm.dmtx_light(frametimes, paradigm, drift_model='cosine',
                         hfcut=128, hrf_model=hrf_model, add_regs=motion,
                         add_reg_names=add_reg_names)


#######################################
# Get the FMRI data
#######################################

fmri_data = surrogate_4d_dataset(shape=shape, n_scans=n_scans)[0]

# if you want to save it as an image
data_file = 'fmri_data.nii'
save(fmri_data, data_file)

########################################
# Perform a GLM analysis
########################################

# GLM fit
Y = fmri_data.get_data().reshape(np.prod(shape), n_scans)
glm = GeneralLinearModel(X)
glm.fit(Y.T)

# specify the contrast [1 -1 0 ..]
contrast = np.zeros(X.shape[1])
contrast[0] = 1
contrast[1] = - 1

# compute the constrast image related to it
zvals = glm.contrast(contrast).z_score()
contrast_image = Nifti1Image(np.reshape(zvals, shape), affine)

# if you want to save the contrast as an image
contrast_path = 'zmap.nii'
save(contrast_image, contrast_path)

print('Wrote the some of the results as images in directory %s' %
      op.abspath(os.getcwd()))

h, c = np.histogram(zvals, 100)

# Show the histogram
plt.figure()
plt.bar(c[: - 1], h, width=.1)
plt.title(' Histogram of the z-values')
plt.show()
